#include <QtGui>
#include <QPixmap>
#include <QLabel>
#include <iostream>
#include "gough_stewart_manager_gui.moc"
#include "gough_stewart_manager_gui.h"

#include <qapplication.h>
#include <qwt_plot.h>
#include <qwt_plot_marker.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_grid.h>
#include <qwt_symbol.h>
#include <qwt_legend.h>

#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qmainwindow.h>

#include <qwidget.h>
#include <mgl/mgl_c.h>

#include <mgl/mgl.h>
#include <mgl/mgl_qt.h>


CGOUGH_STEWART_MANAGER::CGOUGH_STEWART_MANAGER()
{   
    QPalette pal;
    QString qSliderPos, qSliderVel;
    int serialNo,version,numMotors;
    int ControllerSerial,ControllerVersion,ControllerNumServos;
    const char* DeviceType;
    int result,i;
    const char *err;
    QPixmap  pixmap("../src/gui/pictures/vikini.png");
    QPixmap  pixmap_logo("../src/gui/pictures/logo_iri.png");
    std::string cont_name="Phidget1061";
    this->labelImage = new QLabel(this);
    this->logo = new QLabel(this);
    this->labelImage->setGeometry(QRect(320, 130, 220, 220));
    this->logo->setGeometry(QRect(330, 30, 200, 100));
    this->labelImage->setPixmap(pixmap);
    this->logo->setPixmap(pixmap_logo);
    this->labelImage->setScaledContents(1) ;
    this->logo->setScaledContents(1) ;
    this->SensorsEnabled=false;
    setupUi(this); 
//First tab
    connect(ServoChannel,SIGNAL(valueChanged(int)),this,SLOT(ServoChannel_slot(int)));
    connect(SliderTargetPosition,SIGNAL(valueChanged(int)),this,SLOT(targetPosition_slot(int)));
    connect(SliderVelocity,SIGNAL(valueChanged(int)),this,SLOT(Velocity_slot(int)));    
    connect(ButtonStopSingle,SIGNAL(clicked()),this,SLOT(Stop_single_slot()));
    connect(ButtonMove,SIGNAL(clicked()),this,SLOT(Move_slot()));
    connect(ButtonResetValues,SIGNAL(clicked()),this,SLOT(ResetValues_slot()));
//Second tab
    connect(ButtonUp,SIGNAL(clicked()),this,SLOT(Up_slot()));
    connect(ButtonDown,SIGNAL(clicked()),this,SLOT(Down_slot()));
    connect(ButtonHome,SIGNAL(clicked()),this,SLOT(Home_slot()));
    connect(ButtonStop,SIGNAL(clicked()),this,SLOT(Stop_slot()));
    connect(ButtonBars1,SIGNAL(clicked()),this,SLOT(Bars1_slot()));
    connect(ButtonBars2,SIGNAL(clicked()),this,SLOT(Bars2_slot()));
//Third tab
    connect(ButtonLine,SIGNAL(clicked()),this,SLOT(Line_slot())); 
    connect(ButtonSquare,SIGNAL(clicked()),this,SLOT(Square_slot()));    
    connect(ButtonCircle,SIGNAL(clicked()),this,SLOT(Circle_slot()));        
    connect(ButtonHelix,SIGNAL(clicked()),this,SLOT(Helix_slot()));   
    connect(ButtonRotx,SIGNAL(clicked()),this,SLOT(Rotx_slot()));   
    connect(ButtonRoty,SIGNAL(clicked()),this,SLOT(Roty_slot()));   
    connect(ButtonRotz,SIGNAL(clicked()),this,SLOT(Rotz_slot()));   
    connect(ButtonRotxy,SIGNAL(clicked()),this,SLOT(Rotxy_slot())); 
    connect(ButtonOpenGraph, SIGNAL(clicked()),this,SLOT(Graf_slot()));
    connect(ButtonCloseAll,SIGNAL(clicked()),this,SLOT(CloseAll_slot()));
    connect(checkBoxSensors,SIGNAL(stateChanged(int)),this,SLOT(UseSensors_slot(int)));


    this->SensorState[0]=1; 
    this->SensorState[1]=1;
    this->SensorState[2]=1;
    this->controller = new CPHIDGET1061(cont_name);
    this->event_server= CEventServer::instance();
    this->IDbeta[0]=IDbeta1;
    this->IDbeta[1]=IDbeta2;
    this->IDbeta[2]=IDbeta3;
    this->p=57.0;
    this->P=102.0;
    this->deltha=10.0;
    this->channel=0;
    this->iter=1000;
    this->error=-1;
    this->ms=-1;
    

#ifndef _HAVE_XSD
    this->config.position.resize(6);
    this->config.velocity.resize(6);
    this->config.acceleration.resize(6);
    this->config.encoder_resolution.resize(6);
    this->config.gear_factor.resize(6);
    for (k=0;k<6;k++)
    {
    this->config.position[k].max=255.1;
    this->config.position[k].min=0.0;
    this->config.encoder_resolution[k]=1;
    this->config.gear_factor[k]=1;
    }
#endif
    this->position.resize(6);
    this->velocity.resize(6);
    this->accel.resize(6);
    this->enc_res.resize(6);
    this->gear_ratio.resize(6);
    this->enable.resize(6);
    this->relative.resize(6);
    this->endefector.resize(3);
    this->rot.resize(12);
    this->m.resize(6);
    this->beta.resize(3);
    this->orient_angles.resize(3);
    this->event_id=-1;

    for (k=0;k<6;k++)
  	{ 
 	this->relative[k]=true;
	this->enable[k]=true;
	this->position[k]=255.0/2;
	this->velocity[k]=15.0;
	this->accel[k]=100.0;
	}
	// Init spatatial phidgets
		for (i=0;i<3;i++)
		{ 
		 this->beta_sensor[i].open(IDbeta[i]);
		 this->beta_sensor[i].wait(1000);
		 this->beta_sensor[i].displayProperties();
		 this->beta_sensor[i].setDataRate(100); 
		}
  this->controller->open();    // Opening servo phidget
  this->controller->wait(1000);
  this->controller->displayProperties();

  #ifdef _HAVE_XSD
  this->controller->load_config(this->config_file);
  this->events.push_back(this->controller.get_position_feedback_event());
  #else
  this->controller->config(&this->config);
  events.push_back(this->controller->config_position_feedback(fb_polling,100.0));
  #endif
  this->controller->set_absolute_motion(this->relative);
  this->controller->enable(this->enable); 
  this->controller->enable_position_feedback();
  this->event_id=this->event_server->wait_first(this->events);
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) 
   {
    std::cout << " Home reached" << std::endl;   
    this->UpdateInfo();
    }

    if(this->controller->attached)
      {   
       this->AttachedText->setText("True"); 
       for (i=0;i<this->controller->numMotors-2;i++)
	     {
        this->SlidersInfo[i].targetPos=(8050+5200)/2;
        this->SlidersInfo[i].setVel=0;
        this->current.push_back(0.0);
       }
      this->SliderTargetPosition->setValue(this->SlidersInfo[0].targetPos);
      sprintf(this->buf,"%.2f",((float)this->SliderTargetPosition->sliderPosition()/100));
      this->SetPositionText->setText(this->buf);
      this->SliderVelocity->setValue(this->SlidersInfo[0].setVel);
      sprintf(this->buf,"%.2f",((float)this->SliderVelocity->sliderPosition()/100));
      this->VelocityText->setText(this->buf);      
	    this->ServoChannel->setValue(0);
      this->channel=0;
      sprintf(this->buf,"%d",this->controller->serialNo);
      this->SerialText->setText(this->buf);
      sprintf(this->buf,"%d",this->controller->version);
      this->VersionText->setText(this->buf);
      sprintf(this->buf,"%d",this->controller->numMotors);
      this->NumServoText->setText(this->buf);
      sprintf(this->buf,"%s",this->controller->DeviceType);
      this->NameText->setText(this->buf);
       for (i=0;i<6;i++)
	     {
        this->SlidersInfo[i].targetPos=(8050+5200)/2;
        this->SlidersInfo[i].setVel=2500;
       }     
     this->SliderTargetPosition->setValue(this->SlidersInfo[0].targetPos);
     sprintf(this->buf,"%.2f",((float)this->SliderTargetPosition->sliderPosition()/100));
     this->SetPositionText->setText(this->buf);
     this->SliderVelocity->setValue(this->SlidersInfo[0].setVel);
     sprintf(this->buf,"%.2f",((float)this->SliderVelocity->sliderPosition()/100));
     this->VelocityText->setText(this->buf);
      }
     if(this->beta_sensor[0].attached)
       {
       this->S1AttachedText->setText("True");     
       sprintf(this->buf,"%s",this->beta_sensor[0].DeviceType);
       this->S1NameText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[0].serialNo);
       this->S1SerialText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[0].version);
       this->S1VersionText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[0].numAccelAxes);
			 this->S1AccText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[0].numGyroAxes);
       this->S1GyroText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[0].numCompassAxes);
       this->S1CompassText->setText(this->buf);
       }
     if(this->beta_sensor[1].attached)
       {
       this->S2AttachedText->setText("True");     
       sprintf(this->buf,"%s",this->beta_sensor[1].DeviceType);
       this->S2NameText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[1].serialNo);
       this->S2SerialText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[1].version);
       this->S2VersionText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[1].numAccelAxes);
			 this->S2AccText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[1].numGyroAxes);
       this->S2GyroText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[1].numCompassAxes);
       this->S2CompassText->setText(this->buf);
       }      
     if(this->beta_sensor[2].attached)
       {
       this->S3AttachedText->setText("True");     
       sprintf(this->buf,"%s",this->beta_sensor[2].DeviceType);
       this->S3NameText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[2].serialNo);
       this->S3SerialText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[2].version);
       this->S3VersionText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[2].numAccelAxes);
			 this->S3AccText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[2].numGyroAxes);
       this->S3GyroText->setText(this->buf);
       sprintf(this->buf,"%d",this->beta_sensor[2].numCompassAxes);
       this->S3CompassText->setText(this->buf);
       }     
  //getchar();
  this->startTimer(10);
}

void CGOUGH_STEWART_MANAGER::timerEvent(QTimerEvent * event)
{
    QPalette pal;
    QString qSliderMaxPos, qSliderMinPos, qSliderPos, qSliderVel,qSliderAcc;
    QString qactualVel, qactualPos,qactualCurrent;
    int serialNo,version,numMotors;
    double actualVel, actualPos,actualCurrent;
    const char* DeviceType;

        //Update velocity, position and current
 if(this->tabWidgetBasicMotionControl->currentIndex() ==0)
    {   
      this->channel= this->ServoChannel->value(); 
      this->controller->get_position();
      sprintf(this->buf,"%.2f",(466.0526-this->position[this->channel])/5.7894);
      sprintf(this->buf,"%.2f",(720.2831-this->position[this->channel])/8.9474);
      this->ActualPosicionText->setText(this->buf);
      this->controller->get_velocity();
      sprintf(this->buf,"%.2f",this->velocity[this->channel]);
      this->ActualVelocityText->setText(this->buf);
      this->current=this->controller->get_current();
      sprintf(this->buf,"%.2f",this->current[this->channel]);
      this->CurrentText->setText(this->buf);     
      this->SlidersInfo[this->channel].targetPos=this->SliderTargetPosition->value();
      this->SliderTargetPosition->setValue(this->SlidersInfo[this->channel].targetPos);
      this->SlidersInfo[this->channel].setVel=this->SliderVelocity->value();
      this->SliderVelocity->setValue(this->SlidersInfo[this->channel].setVel);
    } 
   this->UpdateInfo();
   update();
}

void CGOUGH_STEWART_MANAGER::Up_slot(void)
{
	int k;
	std::cout << "Going to UP position...";
  for (k=0;k<6;k++)
  { 
  this->position[k]=0.0;
  this->velocity[k]=30.0;
  this->accel[k]=100.0;
  }
  this->event_id=this->event_server->wait_first(this->events);
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) 
   {
    std::cout << " Done" << std::endl;
    std::cout << " UP position reached" << std::endl;   
    this->UpdateInfo();
    }
}

void CGOUGH_STEWART_MANAGER::Down_slot(void)
{
	int k;
	std::cout << "Going to DOWN position...";
  for (k=0;k<6;k++)
  { 
  this->position[k]=255.0;
  this->velocity[k]=30.0;
  this->accel[k]=100.0;
  }
  this->event_id=this->event_server->wait_first(this->events);
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) 
   {
    std::cout << " Done" << std::endl;
    std::cout << " DOWN position reached" << std::endl;   
    this->UpdateInfo();
    }
}

void CGOUGH_STEWART_MANAGER::Home_slot(void)
{
	int k;
	std::cout << "Going to HOME position...";
  for (k=0;k<6;k++)
  { 
  this->position[k]=255.0/2.0;
  this->velocity[k]=30.0;
  this->accel[k]=100.0;
  }
  this->event_id=this->event_server->wait_first(this->events); 
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) 
   {
    std::cout << " Done" << std::endl;
    std::cout << " HOME position reached" << std::endl;   
    this->UpdateInfo();
    }
}
void CGOUGH_STEWART_MANAGER::Stop_slot(void)
{
	std::cout << "STOPPING motors...";
  this->controller->stop();
  std::cout << " Stopped" << std::endl;    
  this->UpdateInfo();
}

void CGOUGH_STEWART_MANAGER::Bars1_slot(void)
{
  int k=0;
	std::cout << "1 by 1 bars moving...";
  for (k=0;k<6;k++)
  { 
  this->position[k]=255.0/2.0;
  this->velocity[k]=50.0;
  this->accel[k]=100.0;
  }
  this->event_id=this->event_server->wait_first(this->events);
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) 
    std::cout << " HOME position reached" << std::endl;   
  for(k=0;k<6;k++)
  {
  this->event_id=this->event_server->wait_first(this->events);
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) 
  this->position[k]=255.0/2.0 + 20;
  sleep(1.5);
  }
  for(k=0;k<6;k++)
  {
  this->event_id=this->event_server->wait_first(this->events);
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) 
  this->position[k]=255.0/2.0 -20;
  sleep(1.5);
  }
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) 
  std::cout << "Finished" << std::endl; 
  this->UpdateInfo();  
}

void CGOUGH_STEWART_MANAGER::Bars2_slot(void)
{
 	std::cout << "2 by 2 bars moving...";
  for (k=0;k<6;k++)
  { 
  this->position[k]=255.0/2.0;
  this->velocity[k]=15.0;
  this->accel[k]=100.0;
  }
  this->event_id=this->event_server->wait_first(this->events);
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) 
    std::cout << " HOME position reached" << std::endl;   
  for(k=0;k<3;k++)
  {
  this->event_id=this->event_server->wait_first(this->events);
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) {
  this->position[2*k]=255.0/2.0 +  30;
  this->position[2*k+1]=255.0/2.0 + 30;}
  sleep(2);
  }
  for(k=0;k<3;k++)
  {
  this->event_id=this->event_server->wait_first(this->events);
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) {
  this->position[2*k]=255.0/2.0 -  30;
  this->position[2*k+1]=255.0/2.0 -  30;}
  sleep(2);
  }
  this->controller->load(this->position,this->velocity,this->accel);
  this->controller->move();
  if(this->event_id==0) 
  std::cout << "Finished" << std::endl; 
  this->UpdateInfo();
}

void CGOUGH_STEWART_MANAGER::ServoChannel_slot(int channel)
{
this->channel=channel;
this->SliderTargetPosition->setValue(this->SlidersInfo[this->channel].targetPos);
this->SliderVelocity->setValue(this->SlidersInfo[this->channel].setVel);
}

void  CGOUGH_STEWART_MANAGER::targetPosition_slot(int pos) //Slider
{
sprintf(this->buf,"%.2f",(float)pos/100);
this->SetPositionText->setText(this->buf);
this->position[this->channel]=(float)pos/100;
}

void CGOUGH_STEWART_MANAGER::Velocity_slot(int vel)
{
sprintf(this->buf,"%.2f",(float)vel/100);
this->VelocityText->setText(this->buf);
this->velocity[this->channel] =(float)vel/100;
}

void CGOUGH_STEWART_MANAGER::Move_slot(void)
{
int i=0;
std::vector<float> newpos,newvel;
newpos.resize(6);
newvel.resize(6);
for(i=0;i<6;i++)
{
//newpos[i]=466.0526-(this->SlidersInfo[i].targetPos/100)*5.7894;
newpos[i]=720.2831-(this->SlidersInfo[i].targetPos/100)*8.9474;
newvel[i]=this->SlidersInfo[i].setVel/100;
}
this->event_id=this->event_server->wait_first(this->events);
this->controller->load(newpos,newvel,this->accel);
this->controller->move();
  if(this->event_id==0) 
   {
    std::cout << " Done" << std::endl;
    std::cout << " Position reached" << std::endl;   
    this->UpdateInfo();
    }
}

void CGOUGH_STEWART_MANAGER::Stop_single_slot(void)
{
this->controller->stop();
this->UpdateInfo();
}

void CGOUGH_STEWART_MANAGER::ResetValues_slot(void)
{
       for (i=0;i<6;i++)
	     {
        this->SlidersInfo[i].targetPos=(8050+5200)/2;
        this->SlidersInfo[i].setVel=2500;
       }     
     this->SliderTargetPosition->setValue(this->SlidersInfo[0].targetPos);
     sprintf(this->buf,"%.2f",((float)this->SliderTargetPosition->sliderPosition()/100));
     this->SetPositionText->setText(this->buf);
     this->SliderVelocity->setValue(this->SlidersInfo[0].setVel);
     sprintf(this->buf,"%.2f",((float)this->SliderVelocity->sliderPosition()/100));
     this->VelocityText->setText(this->buf);
}

void CGOUGH_STEWART_MANAGER::Line_slot(void)
{
	int k,l;
	std::cout << "Line trajectory..";
  for(l=0;l<NumSamples;l++)
   { 
    this->event_id=this->event_server->wait_first(this->events);
    if(this->event_id==0) //Ha arribat a la posició
    {
    for(k=0;k<6;k++)
     { 
      this->position[k]=trajectory_line[l][k]; // carreguem la seguent
      if(l!=0) this->velocity[k]=speed_line[l-1][k];  else velocity[k]=15.0;
      this->accel[k]=3*ACC_RESOLUTION;   
     }
    this->controller->load(position,velocity,accel);
    this->controller->move();
    //Display de les 6 longituds
   }
   this->UpdateInfo();
   //Guardar dades pels grafics: 6longituds, 3angles sensors, posicio xyz i orientacions
 for(k=0;k<6;k++){
 this->Length_experimental[l][k]=this->m[k];
 this->Length_teoric[l][k]=Length_teoric_line[l][k];
}
 for(k=0;k<3;k++)
 {
 this->Beta_experimental[l][k]=this->beta[k];
 this->Beta_teoric[l][k]=Beta_teoric_line[l][k];
 this->Pos_experimental[l][k]=this->endefector[k];
 this->Pos_teoric[l][k]=Pos_teoric_line[l][k];
 this->Orient_experimental[l][k]=this->orient_angles[k];
 this->Orient_teoric[l][k]=Orient_teoric_line[l][k];
 this->Pos_error[l][k]=Pos_teoric_line[l][k]-this->endefector[k];
 this->Orient_error[l][k]=Orient_teoric_line[l][k]-this->orient_angles[k]; 
 } 
 }
  this->beta.clear();
  this->m.clear();
  this->rot.clear();
  this->endefector.clear();
}

void CGOUGH_STEWART_MANAGER::Square_slot(void)
{
  int k,l;
  std::cout << "Square trajectory..";

  for(l=0;l<NumSamples;l++)
   { 
    this->event_id=this->event_server->wait_first(this->events);
    if(this->event_id==0) //Ha arribat a la posició
    {
    for(k=0;k<6;k++)
     { 
      this->position[k]=trajectory_square[l][k]; // carreguem la seguent
      if(l!=0) this->velocity[k]=speed_square[l-1][k];  else velocity[k]=15.0;
      this->accel[k]=3*ACC_RESOLUTION;   
     }
    this->controller->load(position,velocity,accel);
    this->controller->move();
    //Display de les 6 longituds
    //for(k=0;k<3;k++) this->beta.push_back(Beta_teoric_square[l][k]); //Fixar els valors dels angles a teorics
    this->UpdateInfo();
   //Guardar dades pels grafics: 6longituds, 3angles sensors, posicio xyz i orientacions
 for(k=0;k<6;k++)
 {
 this->Length_experimental[l][k]=this->m[k];
 this->Length_teoric[l][k]=Length_teoric_square[l][k];
 }
 for(k=0;k<3;k++)
 {
 this->Beta_experimental[l][k]=this->beta[k];
 this->Beta_teoric[l][k]=Beta_teoric_square[l][k];
 this->Pos_experimental[l][k]=this->endefector[k];
 this->Pos_teoric[l][k]=Pos_teoric_square[l][k];
 this->Orient_experimental[l][k]=this->orient_angles[k];
 this->Orient_teoric[l][k]=Orient_teoric_square[l][k];
 this->Pos_error[l][k]=Pos_teoric_square[l][k]-this->endefector[k];
 this->Orient_error[l][k]=Orient_teoric_square[l][k]-this->orient_angles[k]; 
 } 
}
this->nr[l][0]=this->iter;
this->nr[l][1]=this->error;
this->nr[l][2]=this->ms;
}
  this->beta.clear();
  this->m.clear();
  this->rot.clear();
  this->endefector.clear();
}

void CGOUGH_STEWART_MANAGER::Circle_slot(void)
{
	int k,l;
	std::cout << "Circle trajectory..";
  for(l=0;l<NumSamples;l++)
   { 
    this->event_id=this->event_server->wait_first(this->events);
    if(this->event_id==0) //Ha arribat a la posició
    {
    for(k=0;k<6;k++)
     { 
      this->position[k]=trajectory_circle[l][k]; // carreguem la seguent
      if(l!=0) this->velocity[k]=speed_circle[l-1][k];  else velocity[k]=15.0;
      this->accel[k]=3*ACC_RESOLUTION;   
     }
    this->controller->load(position,velocity,accel);
    this->controller->move();
    //Display de les 6 longituds
    this->UpdateInfo();    
   }
   //Guardar dades pels grafics: 6longituds, 3angles sensors, posicio xyz i orientacions
 for(k=0;k<6;k++){
 this->Length_experimental[l][k]=this->m[k];
 this->Length_teoric[l][k]=Length_teoric_circle[l][k];}
 for(k=0;k<3;k++)
 {
 this->Beta_experimental[l][k]=this->beta[k];
 this->Beta_teoric[l][k]=Beta_teoric_circle[l][k];
 this->Pos_experimental[l][k]=this->endefector[k];
 this->Pos_teoric[l][k]=Pos_teoric_circle[l][k];
 this->Orient_experimental[l][k]=this->orient_angles[k];
 this->Orient_teoric[l][k]=Orient_teoric_circle[l][k];
 this->Pos_error[l][k]=Pos_teoric_circle[l][k]-this->endefector[k];
 this->Orient_error[l][k]=Orient_teoric_circle[l][k]-this->orient_angles[k]; 
 } 
 }
  this->beta.clear();
  this->m.clear();
  this->rot.clear();
  this->endefector.clear();
}

void CGOUGH_STEWART_MANAGER::Helix_slot(void)
{
int k,l;
std::cout << "Circle trajectory..";
  for(l=0;l<NumSamples;l++)
   { 
    this->event_id=this->event_server->wait_first(this->events);
    if(this->event_id==0) //Ha arribat a la posició
    {
    for(k=0;k<6;k++)
     { 
      this->position[k]=trajectory_helix[l][k]; // carreguem la seguent
      if(l!=0) this->velocity[k]=speed_helix[l-1][k];  else velocity[k]=15.0;
      this->accel[k]=3*ACC_RESOLUTION;   
     }
    this->controller->load(position,velocity,accel);
    this->controller->move();
    //Display de les 6 longituds
    this->UpdateInfo();    
   }
   //Guardar dades pels grafics: 6longituds, 3angles sensors, posicio xyz i orientacions
 for(k=0;k<6;k++){
 this->Length_experimental[l][k]=this->m[k];
 this->Length_teoric[l][k]=Length_teoric_helix[l][k];}
 for(k=0;k<3;k++)
 {
 this->Beta_experimental[l][k]=this->beta[k];
 this->Beta_teoric[l][k]=Beta_teoric_helix[l][k];
 this->Pos_experimental[l][k]=this->endefector[k];
 this->Pos_teoric[l][k]=Pos_teoric_helix[l][k];
 this->Orient_experimental[l][k]=this->orient_angles[k];
 this->Orient_teoric[l][k]=Orient_teoric_helix[l][k];
 this->Pos_error[l][k]=Pos_teoric_helix[l][k]-this->endefector[k];
 this->Orient_error[l][k]=Orient_teoric_helix[l][k]-this->orient_angles[k]; 
 } 
 }
  this->beta.clear();
  this->m.clear();
  this->rot.clear();
  this->endefector.clear();
}

void CGOUGH_STEWART_MANAGER::Rotx_slot(void)
{
  int k,l;
  std::cout << "Square X Rotated trajectory..";

  for(l=0;l<NumSamples;l++)
   { 
    this->event_id=this->event_server->wait_first(this->events);
    if(this->event_id==0) //Ha arribat a la posició
    {
    for(k=0;k<6;k++)
     { 
      this->position[k]=trajectory_rotX[l][k]; // carreguem la seguent
      if(l!=0) this->velocity[k]=speed_rotX[l-1][k];  else velocity[k]=15.0;
      this->accel[k]=3*ACC_RESOLUTION;   
     }
    this->controller->load(position,velocity,accel);
    this->controller->move();
    //Display de les 6 longituds
    //for(k=0;k<3;k++) this->beta.push_back(Beta_teoric_square[l][k]); //Fixar els valors dels angles a teorics
    this->UpdateInfo();
   //Guardar dades pels grafics: 6longituds, 3angles sensors, posicio xyz i orientacions
 for(k=0;k<6;k++)
 {
 this->Length_experimental[l][k]=this->m[k];
 this->Length_teoric[l][k]=Length_teoric_square_rotX[l][k];
 }
 for(k=0;k<3;k++)
 {
 this->Beta_experimental[l][k]=this->beta[k];
 this->Beta_teoric[l][k]=Beta_teoric_square_rotX[l][k];
 this->Pos_experimental[l][k]=this->endefector[k];
 this->Pos_teoric[l][k]=Pos_teoric_square_rotX[l][k];
 this->Orient_experimental[l][k]=this->orient_angles[k];
 this->Orient_teoric[l][k]=Orient_teoric_square_rotX[l][k];
 this->Pos_error[l][k]=Pos_teoric_square_rotX[l][k]-this->endefector[k];
 this->Orient_error[l][k]=Orient_teoric_square_rotX[l][k]-this->orient_angles[k]; 
 } 
}
 }
  this->beta.clear();
  this->m.clear();
  this->rot.clear();
  this->endefector.clear();
}

void CGOUGH_STEWART_MANAGER::Roty_slot(void)
{
  int k,l;
  std::cout << "Square Y Rotated trajectory..";

  for(l=0;l<NumSamples;l++)
   { 
    this->event_id=this->event_server->wait_first(this->events);
    if(this->event_id==0) //Ha arribat a la posició
    {
    for(k=0;k<6;k++)
     { 
      this->position[k]=trajectory_rotY[l][k]; // carreguem la seguent
      if(l!=0) this->velocity[k]=speed_rotY[l-1][k];  else velocity[k]=15.0;
      this->accel[k]=3*ACC_RESOLUTION;   
     }
    this->controller->load(position,velocity,accel);
    this->controller->move();
    //Display de les 6 longituds
    //for(k=0;k<3;k++) this->beta.push_back(Beta_teoric_square[l][k]); //Fixar els valors dels angles a teorics
    this->UpdateInfo();
   //Guardar dades pels grafics: 6longituds, 3angles sensors, posicio xyz i orientacions
 for(k=0;k<6;k++)
 {
 this->Length_experimental[l][k]=this->m[k];
 this->Length_teoric[l][k]=Length_teoric_square_rotY[l][k];
 }
 for(k=0;k<3;k++)
 {
 this->Beta_experimental[l][k]=this->beta[k];
 this->Beta_teoric[l][k]=Beta_teoric_square_rotY[l][k];
 this->Pos_experimental[l][k]=this->endefector[k];
 this->Pos_teoric[l][k]=Pos_teoric_square_rotY[l][k];
 this->Orient_experimental[l][k]=this->orient_angles[k];
 this->Orient_teoric[l][k]=Orient_teoric_square_rotY[l][k];
 this->Pos_error[l][k]=Pos_teoric_square_rotY[l][k]-this->endefector[k];
 this->Orient_error[l][k]=Orient_teoric_square_rotY[l][k]-this->orient_angles[k]; 
 } 
}
 }
  this->beta.clear();
  this->m.clear();
  this->rot.clear();
  this->endefector.clear();
}

void CGOUGH_STEWART_MANAGER::Rotz_slot(void)
{
  int k,l;
  std::cout << "Square Z Rotated trajectory..";

  for(l=0;l<NumSamples;l++)
   { 
    this->event_id=this->event_server->wait_first(this->events);
    if(this->event_id==0) //Ha arribat a la posició
    {
    for(k=0;k<6;k++)
     { 
      this->position[k]=trajectory_rotZ[l][k]; // carreguem la seguent
      if(l!=0) this->velocity[k]=speed_rotZ[l-1][k];  else velocity[k]=15.0;
      this->accel[k]=3*ACC_RESOLUTION;   
     }
    this->controller->load(position,velocity,accel);
    this->controller->move();
    //Display de les 6 longituds
    //for(k=0;k<3;k++) this->beta.push_back(Beta_teoric_square[l][k]); //Fixar els valors dels angles a teorics
    this->UpdateInfo();
   //Guardar dades pels grafics: 6longituds, 3angles sensors, posicio xyz i orientacions
 for(k=0;k<6;k++)
 {
 this->Length_experimental[l][k]=this->m[k];
 this->Length_teoric[l][k]=Length_teoric_square_rotZ[l][k];
 }
 for(k=0;k<3;k++)
 {
 this->Beta_experimental[l][k]=this->beta[k];
 this->Beta_teoric[l][k]=Beta_teoric_square_rotZ[l][k];
 this->Pos_experimental[l][k]=this->endefector[k];
 this->Pos_teoric[l][k]=Pos_teoric_square_rotZ[l][k];
 this->Orient_experimental[l][k]=this->orient_angles[k];
 this->Orient_teoric[l][k]=Orient_teoric_square_rotZ[l][k];
 this->Pos_error[l][k]=Pos_teoric_square_rotZ[l][k]-this->endefector[k];
 this->Orient_error[l][k]=Orient_teoric_square_rotZ[l][k]-this->orient_angles[k]; 
 } 
}
 }
  this->beta.clear();
  this->m.clear();
  this->rot.clear();
  this->endefector.clear();
}

void CGOUGH_STEWART_MANAGER::Rotxy_slot(void)
{
  int k,l;
  std::cout << "Square XY Rotated trajectory..";

  for(l=0;l<NumSamples;l++)
   { 
    this->event_id=this->event_server->wait_first(this->events);
    if(this->event_id==0) //Ha arribat a la posició
    {
    for(k=0;k<6;k++)
     { 
      this->position[k]=trajectory_rotXY[l][k]; // carreguem la seguent
      if(l!=0) this->velocity[k]=speed_rotXY[l-1][k];  else velocity[k]=15.0;
      this->accel[k]=3*ACC_RESOLUTION;   
     }
    this->controller->load(position,velocity,accel);
    this->controller->move();
    //Display de les 6 longituds
    //for(k=0;k<3;k++) this->beta.push_back(Beta_teoric_square[l][k]); //Fixar els valors dels angles a teorics
    this->UpdateInfo();
   //Guardar dades pels grafics: 6longituds, 3angles sensors, posicio xyz i orientacions
 for(k=0;k<6;k++)
 {
 this->Length_experimental[l][k]=this->m[k];
 this->Length_teoric[l][k]=Length_teoric_square_rotXY[l][k];
 }
 for(k=0;k<3;k++)
 {
 this->Beta_experimental[l][k]=this->beta[k];
 this->Beta_teoric[l][k]=Beta_teoric_square_rotXY[l][k];
 this->Pos_experimental[l][k]=this->endefector[k];
 this->Pos_teoric[l][k]=Pos_teoric_square_rotXY[l][k];
 this->Orient_experimental[l][k]=this->orient_angles[k];
 this->Orient_teoric[l][k]=Orient_teoric_square_rotXY[l][k];
 this->Pos_error[l][k]=Pos_teoric_square_rotXY[l][k]-this->endefector[k];
 this->Orient_error[l][k]=Orient_teoric_square_rotXY[l][k]-this->orient_angles[k]; 
 } 
}
 }
  this->beta.clear();
  this->m.clear();
  this->rot.clear();
  this->endefector.clear();
}

int CGOUGH_STEWART_MANAGER::Graf_slot(void)
{
    int k=0;
    QwtPlot *plotL[6],*plotB[3],*plotP[3],*plotO[3],*plotPosError,*plotOrientError;//= new QwtPlot();
    QwtPlotGrid *gridL[6],*gridB[3],*gridP[3],*gridO[3],*gridPosError,*gridOrientError;
    QwtPlotCurve *curveExpL[6],*curveTL[6],*curveExpB[3],*curveTB[3],*curveExpP[3],*curveTP[3],*curveExpO[3],*curveTO[3],*curvePosError[3],*curveOrientError[3];
    QPolygonF pointsExpL[6],pointsTL[6],pointsExpB[3],pointsTB[3],pointsExpP[3],pointsTP[3],pointsExpO[3],pointsTO[3],pointsPosError[3],pointsOrientError[3];
    QPen pen[6];
    pen[0]=QPen(Qt::blue,2);
    pen[1]=QPen(Qt::red,2);
    pen[2]=QPen(Qt::green,2);
    pen[3]=QPen(Qt::magenta,2);
    pen[4]=QPen(Qt::cyan,2);
    pen[5]=QPen(Qt::yellow,2);
    //Plot Length graphics
   for(k=0;k<6;k++) 
    {
    plotL[k]= new QwtPlot();
    this->plots[k]=plotL[k];//mem id plot for closing
    sprintf(this->buf,"Bar Length %d [cm]",k+1);
    plotL[k]->setTitle(this->buf);
    plotL[k]->setCanvasBackground( Qt::white );
    plotL[k]->setAxisScale( QwtPlot::yLeft, 50.0, 80.0 );
    plotL[k]->insertLegend( new QwtLegend() );
    gridL[k] = new QwtPlotGrid();
    gridL[k]->attach(plotL[k]);
    curveExpL[k]=new QwtPlotCurve();
    curveExpL[k]->setTitle("Experimental");
    curveExpL[k]->setPen(pen[0]);
    curveExpL[k]->setRenderHint( QwtPlotItem::RenderAntialiased, true );
    curveTL[k]=new QwtPlotCurve();
    curveTL[k]->setTitle("Theoric");
    curveTL[k]->setPen(pen[1]);
    curveTL[k]->setRenderHint( QwtPlotItem::RenderAntialiased, true );
    }
    for(unsigned int l=0;l<NumSamples;l++)
      for(unsigned int k=0;k<6;k++)
      {
      pointsTL[k] << QPointF(l,this->Length_teoric[l][k]); 
      pointsExpL[k] << QPointF(l,this->Length_experimental[l][(k+5)%6]);   
     }    
    for(unsigned int k=0;k<6;k++){
    curveExpL[k]->setSamples( pointsExpL[k]);
    curveExpL[k]->attach( plotL[k] );
    curveTL[k]->setSamples( pointsTL[k]);
    curveTL[k]->attach( plotL[k] );    
    plotL[k]->resize( 560, 165 );
    plotL[k]->move(0,165*k);
    plotL[k]->show();
    }
    char coord[3]={'X','Y','Z'};
    for(k=0;k<3;k++) 
    {
    plotB[k]= new QwtPlot();
    plotP[k]= new QwtPlot();
    plotO[k]= new QwtPlot();
    sprintf(this->buf,"Beta Angle %d [rad]",k+1);
    plotB[k]->setTitle(this->buf);
    sprintf(this->buf,"Endefector %c Coordinate [cm]",coord[k]);
    plotP[k]->setTitle(this->buf);
    sprintf(this->buf,"Rotation in %c Axis [rad]",coord[k]);
    plotO[k]->setTitle(this->buf);   

    plotB[k]->setCanvasBackground( Qt::white );
    plotB[k]->setAxisScale( QwtPlot::yLeft, 0.0, 1.5 );
    plotB[k]->insertLegend( new QwtLegend() );
    gridB[k] = new QwtPlotGrid();
    gridB[k]->attach(plotB[k]);
    plotP[k]->setCanvasBackground( Qt::white );
    plotP[k]->setAxisScale( QwtPlot::yLeft, 0.0, 80.0 );
    plotP[k]->insertLegend( new QwtLegend() );
    gridP[k] = new QwtPlotGrid();
    gridP[k]->attach(plotP[k]);
    plotO[k]->setCanvasBackground( Qt::white );
    plotO[k]->setAxisScale( QwtPlot::yLeft, -0.4, 0.4 );
    plotO[k]->insertLegend( new QwtLegend() );
    gridO[k] = new QwtPlotGrid();
    gridO[k]->attach(plotO[k]); 

  
    curveExpB[k]=new QwtPlotCurve();
    curveExpB[k]->setTitle("Experimental");
    curveExpB[k]->setPen(pen[0]);
    curveExpB[k]->setRenderHint( QwtPlotItem::RenderAntialiased, true );
    curveTB[k]=new QwtPlotCurve();
    curveTB[k]->setTitle("Theoric");
    curveTB[k]->setPen(pen[1]);
    curveTB[k]->setRenderHint( QwtPlotItem::RenderAntialiased, true );
    curveExpP[k]=new QwtPlotCurve();
    curveExpP[k]->setTitle("Experimental");
    curveExpP[k]->setPen(pen[0]);
    curveExpP[k]->setRenderHint( QwtPlotItem::RenderAntialiased, true );
    curveTP[k]=new QwtPlotCurve();
    curveTP[k]->setTitle("Theoric");
    curveTP[k]->setPen(pen[1]);
    curveTP[k]->setRenderHint( QwtPlotItem::RenderAntialiased, true );
    curveExpO[k]=new QwtPlotCurve();
    curveExpO[k]->setTitle("Experimental");
    curveExpO[k]->setPen(pen[0]);
    curveExpO[k]->setRenderHint( QwtPlotItem::RenderAntialiased, true );
    curveTO[k]=new QwtPlotCurve();
    curveTO[k]->setTitle("Theoric");
    curveTO[k]->setPen(pen[1]);
    curveTO[k]->setRenderHint( QwtPlotItem::RenderAntialiased, true );

    }

    plotPosError= new QwtPlot();
    plotOrientError= new QwtPlot();
    sprintf(this->buf,"Position Error [cm]");
    plotPosError->setTitle(this->buf);   
    sprintf(this->buf,"Rotation Error [rad]");
    plotOrientError->setTitle(this->buf);   
    plotPosError->setCanvasBackground( Qt::white );
    if(this->SensorsEnabled) plotPosError->setAxisScale( QwtPlot::yLeft, -10, 10 );
    else plotPosError->setAxisScale( QwtPlot::yLeft, -1, 1 );
    plotPosError->insertLegend( new QwtLegend() );
    gridPosError = new QwtPlotGrid();
    gridPosError->attach(plotPosError);
    plotOrientError->setCanvasBackground( Qt::white );
    if(this->SensorsEnabled) plotOrientError->setAxisScale( QwtPlot::yLeft, -1, 1 );
    else plotOrientError->setAxisScale( QwtPlot::yLeft, -0.4, 0.4);
    plotOrientError->insertLegend( new QwtLegend() );
    gridOrientError = new QwtPlotGrid();
    gridOrientError->attach(plotOrientError);

    for(k=0;k<3;k++){
    curvePosError[k]=new QwtPlotCurve();
    sprintf(this->buf,"%c coordinate",coord[k]);
    curvePosError[k]->setTitle(this->buf);
    curvePosError[k]->setPen(pen[k]);
    curvePosError[k]->setRenderHint( QwtPlotItem::RenderAntialiased, true );
    curveOrientError[k]=new QwtPlotCurve();
    sprintf(this->buf,"%c rotation",coord[k]);
    curveOrientError[k]->setTitle(this->buf);
    curveOrientError[k]->setPen(pen[k]);
    curveOrientError[k]->setRenderHint( QwtPlotItem::RenderAntialiased, true );
    }

    for(unsigned int l=0;l<NumSamples;l++)
      for(unsigned int k=0;k<3;k++)
      {
      pointsExpB[k] << QPointF(l,this->Beta_experimental[l][k]);
      pointsTB[k] << QPointF(l,this->Beta_teoric[l][k]);    
      pointsExpP[k] << QPointF(l,this->Pos_experimental[l][k]);
      pointsTP[k] << QPointF(l,this->Pos_teoric[l][k]);
      pointsExpO[k] << QPointF(l,this->Orient_experimental[l][k]);
      pointsTO[k] << QPointF(l,this->Orient_teoric[l][k]);
      pointsPosError[k] << QPointF(l,this->Pos_error[l][k]);
      pointsOrientError[k] << QPointF(l,this->Orient_error[l][k]);
     }
    
    for(unsigned int k=0;k<3;k++){
    curveExpB[k]->setSamples( pointsExpB[k]);
    curveExpB[k]->attach( plotB[k] );
    curveTB[k]->setSamples( pointsTB[k]);
    curveTB[k]->attach( plotB[k] );    
    plotB[k]->resize( 560, 165 );
    plotB[k]->move(560,165*k);
    plotB[k]->show();
    }    
    
    for(unsigned int k=0;k<3;k++){
    curveExpP[k]->setSamples( pointsExpP[k]);
    curveExpP[k]->attach( plotP[k] );
    curveTP[k]->setSamples( pointsTP[k]);
    curveTP[k]->attach( plotP[k] );    
    plotP[k]->resize( 560, 175 );
    plotP[k]->move(560,175*(k+3));
    plotP[k]->show();
    }    
  
    for(unsigned int k=0;k<3;k++){
    curveExpO[k]->setSamples( pointsExpO[k]);
    curveExpO[k]->attach( plotO[k] );
    curveTO[k]->setSamples( pointsTO[k]);
    curveTO[k]->attach( plotO[k] );    
    plotO[k]->resize( 560, 175 );
    plotO[k]->move(2*560,175*k);
    plotO[k]->show();
    } 

    for(unsigned int k=0;k<3;k++){
    curvePosError[k]->setSamples( pointsPosError[k]);
    curvePosError[k]->attach( plotPosError);}
    plotPosError->resize( 560, 175 );
    plotPosError->move(2*560,175*3);
    plotPosError->show();
   
    
    for(unsigned int k=0;k<3;k++){
    curveOrientError[k]->setSamples(pointsOrientError[k]);
    curveOrientError[k]->attach( plotOrientError);}
    plotOrientError->resize( 560, 175 );
    plotOrientError->move(2*560,175*4);
    plotOrientError->show();
  
    
    for(k=0;k<6;k++)
    this->plots[k]=plotL[k];
    for(k=0;k<3;k++){
    this->plots[k+6]=plotB[k];
    this->plots[k+9]=plotP[k];
    this->plots[k+12]=plotO[k];
    }
    this->plots[15]=plotPosError;
    this->plots[16]=plotOrientError;

// Plot 3D trajectory
        float xexp[NumSamples],yexp[NumSamples],zexp[NumSamples];
        float xt[NumSamples],yt[NumSamples],zt[NumSamples];        
        mglGraphZB plot3d;
        plot3d.Alpha(true); 
        plot3d.Light(true);
       // plot3d.Light(0,mglPoint(0,0,1));
        mglData x_exp(NumSamples),y_exp(NumSamples),z_exp(NumSamples),x_t(NumSamples),y_t(NumSamples),z_t(NumSamples);
        for(k=0;k<NumSamples;k++)
        {
        xexp[k]=this->Pos_experimental[k][0];
        yexp[k]=this->Pos_experimental[k][1];
        zexp[k]=this->Pos_experimental[k][2];
        xt[k]=this->Pos_teoric[k][0];
        yt[k]=this->Pos_teoric[k][1];
        zt[k]=this->Pos_teoric[k][2];
        }
        x_exp.Set(xexp,NumSamples);
        y_exp.Set(yexp,NumSamples);
        z_exp.Set(zexp,NumSamples);
        x_t.Set(xt,NumSamples);
        y_t.Set(yt,NumSamples);
        z_t.Set(zt,NumSamples);
      plot3d.Axis(mglPoint(30,10,30),mglPoint(70,60,60));
//        plot3d.Axis(mglPoint(40,20,54),mglPoint(60,40,56));
        plot3d.SetTicks('x', 10, 1);  // sets tick step to 0.5
        plot3d.SetTicks('y', 10, 1);
        plot3d.SetTicks('z', 20, 1);
        plot3d.Title("Endefector Trajectory [cm]                              ",0,7);
        plot3d.Rotate(40,60); 
        plot3d.Axis();
        plot3d.Box();
        plot3d.Plot(x_exp,y_exp,z_exp,":#ob");   //plot.Grid();
        plot3d.Plot(x_t,y_t,z_t,"r"); 
        plot3d.AddLegend("Theoric","r");
        plot3d.AddLegend("Experimental","b");
        plot3d.Legend();
        plot3d.WritePNG("./../graf/Example1.png");

  //Use Qt to display the saved plot
  QDialog *dialog=new QDialog();
  this->graph3d = dialog;
  QVBoxLayout *layout=new QVBoxLayout();
  QPixmap *pixmap=new QPixmap("./../graf/Example1.png");
  QLabel *label=new QLabel();
  label->setPixmap(*pixmap);
  layout->addWidget(label);
  dialog->setLayout(layout);
  dialog->show();

  //write data files
ofstream fs_length("../Data_experiments/Length_experimental.txt"); 
ofstream fs_beta("../Data_experiments/Beta_experimental.txt");
ofstream fs_pos("../Data_experiments/Pos_experimental.txt"); 
ofstream fs_orient("../Data_experiments/Orient_experimental.txt"); 
ofstream fs_error_pos("../Data_experiments/Pos_error.txt"); 
ofstream fs_error_orient("../Data_experiments/Orient_error.txt"); 
ofstream fs_nr("../Data_experiments/Newton_rapson_results.txt"); 

for(unsigned int l=0;l<NumSamples;l++)
{ 
  for(unsigned int k=0;k<6;k++)
   {
   fs_length << this->Length_experimental[l][k] << " ";
   }
   fs_length<<endl;
  for(unsigned int k=0;k<3;k++)
   {
   if(this->SensorsEnabled) fs_beta << this->Beta_experimental[l][k] << " ";
   fs_pos << this->Pos_experimental[l][k] << " ";   
   fs_orient << this->Orient_experimental[l][k] << " ";   
   fs_error_pos << this->Pos_error[l][k] << " ";   
   fs_error_orient << this->Orient_error[l][k] << " ";   
   fs_nr<< this->nr[l][k] << " ";   
   }
   fs_beta<<endl;
   fs_pos <<endl;
   fs_orient << endl;
   fs_error_pos << endl;
   fs_error_orient<< endl;
   fs_nr<< endl;
}
 fs_length.close();
 fs_beta.close();
 fs_pos.close();
 fs_orient.close();
 fs_error_pos.close();
 fs_error_orient.close();
} 


void CGOUGH_STEWART_MANAGER::CloseAll_slot(void)
{
 this->graph3d->close();
 for(k=0;k<17;k++)
  this->plots[k]->close();
    
}
void CGOUGH_STEWART_MANAGER::UseSensors_slot(int state)
{
this->SensorsEnabled=!this->SensorsEnabled;
}

void CGOUGH_STEWART_MANAGER::UpdateInfo(void)
{
  std::vector<float> rot,target,out,fk_rot; 
  static int i=0;
  target.push_back(51);
  target.push_back(29.0);
  target.push_back(48.00);
  rot.push_back(1.0);
  rot.push_back(0.0);
  rot.push_back(0.0);
  rot.push_back(0.0);
  rot.push_back(1.0);
  rot.push_back(0.0);
  rot.push_back(0.0);
  rot.push_back(0.0);
  rot.push_back(1.0);
  this->position = this->controller->get_position();  
  this->velocity = this->controller->get_velocity();
  //Transformacio de cm a pwm. Donada una longitud L cal saber quin PWM assignar-li al controlador. Dades: Lmin=52cm es correspon a PWM 165, Lmax= 81.5 cm es correspon a PWM 0. Dibuixant la recta s'obté la següent equació: PWM = -5.7894 Long +466.0526; PWM = A *Long+B on A=-5.7894 i B=466.0526 Aïllant Long = (PWM-B)/A

 for (k=0;k<6;k++)
//   m.push_back((466.0526-position[k])/5.7894); //+ offset[]);
   this->m.push_back((720.2831-position[k])/8.9474); //+ offset[]);
  shift(this->m);
  if(this->SensorsEnabled)
  {
  //Get beta angle
  float mean_angle=0.0;
  for (i=0;i<3;i++)
   {
     this->beta_sensor[i].getAngle(); //Get the value from the sensor
     usleep(100000);
     this->beta.push_back(this->beta_sensor[i].angle); // Read and puch to Beta vec
   }
  out=fk(this->P,this->p,this->deltha,this->m,this->beta);
  //out=fk_circle(this->P,this->p,this->deltha,this->m,this->beta);    
  } 
  else
    out=fk_newton_raphson(this->m,target,rot);
 
  for(i=0;i<3;i++)
  this->endefector[i]=out[i];
  this->iter=out[12];
  this->error=out[13];
  this->ms=out[14];
//Display de les 6 longituds
  sprintf(this->buf,"%.2f",this->m[0]);
  this->LengthText1->setText(this->buf);
  sprintf(this->buf,"%.2f",this->m[1]);
  this->LengthText2->setText(this->buf);
  sprintf(this->buf,"%.2f",this->m[2]);
  this->LengthText3->setText(this->buf);
  sprintf(this->buf,"%.2f",this->m[3]);
  this->LengthText4->setText(this->buf);
  sprintf(this->buf,"%.2f",this->m[4]);
  this->LengthText5->setText(this->buf);
  sprintf(this->buf,"%.2f",this->m[5]);
  this->LengthText6->setText(this->buf);
  //Display dels 3 angles dels sensors
  sprintf(this->buf,"%.2f",this->beta[0]);
  this->BetaText1->setText(this->buf);
  sprintf(this->buf,"%.2f",this->beta[1]);
  this->BetaText2->setText(this->buf);
  sprintf(this->buf,"%.2f",this->beta[2]);
  this->BetaText3->setText(this->buf);
  //Display de la posicio
  sprintf(this->buf,"%.2f",this->endefector[0]);
  this->XText->setText(this->buf);
  sprintf(this->buf,"%.2f",this->endefector[1]);
  this->YText->setText(this->buf);
  sprintf(this->buf,"%.2f",this->endefector[2]);
  this->ZText->setText(this->buf);
 //transformar matriu rot  en 3 angles
 this->orient_angles[0]= -atan2(out[10],out[11]);   
 this->orient_angles[1]= -atan2(-out[9],sqrt(out[3]*out[3]+out[6]*out[6]));   
 this->orient_angles[2]= -atan2(out[6],out[3]);   

  sprintf(this->buf,"%.2f",this->orient_angles[0]);
  this->AlfaText->setText(this->buf);
  sprintf(this->buf,"%.2f",this->orient_angles[1]);
  this->TetaText->setText(this->buf);
  sprintf(this->buf,"%.2f",this->orient_angles[2]);
  this->GammaText->setText(this->buf);

  this->beta.clear();
  this->m.clear();
  this->orient_angles.clear();
  this->endefector.clear();
  }
