#ifndef _VIKINI_MANAGER_GUI_H
#define _VIKINI_MANAGER_GUI_H
 
#include "ui_gough_stewart_manager_gui.h"
#include "phidget21.h"

#include "eventserver.h"
#include "exceptions.h"
#include "phidget1049.h"
#include "phidget1061.h"
#include <list>

#include "operations.h"
#include "kinematics.h"


#include "trajectory_line.h"
#include "velocity_line.h"
#include "Length_teoric_line.h"
#include "Beta_teoric_line.h"
#include "Pos_teoric_line.h"
#include "Orient_teoric_line.h"


#include "trajectory_circle.h"
#include "velocity_circle.h"
#include "Length_teoric_circle.h"
#include "Beta_teoric_circle.h"
#include "Pos_teoric_circle.h"
#include "Orient_teoric_circle.h"

#include "trajectory_square.h"
#include "velocity_square.h"
#include "Length_teoric_square.h"
#include "Beta_teoric_square.h"
#include "Pos_teoric_square.h"
#include "Orient_teoric_square.h"

#include "trajectory_helix.h"
#include "velocity_helix.h"
#include "Length_teoric_helix.h"
#include "Beta_teoric_helix.h"
#include "Pos_teoric_helix.h"
#include "Orient_teoric_helix.h"

#include "trajectory_rotX.h"
#include "velocity_rotX.h"
#include "Length_teoric_square_rotX.h"
#include "Beta_teoric_square_rotX.h"
#include "Pos_teoric_square_rotX.h"
#include "Orient_teoric_square_rotX.h"

#include "trajectory_rotY.h"
#include "velocity_rotY.h"
#include "Length_teoric_square_rotY.h"
#include "Beta_teoric_square_rotY.h"
#include "Pos_teoric_square_rotY.h"
#include "Orient_teoric_square_rotY.h"

#include "trajectory_rotZ.h"
#include "velocity_rotZ.h"
#include "Length_teoric_square_rotZ.h"
#include "Beta_teoric_square_rotZ.h"
#include "Pos_teoric_square_rotZ.h"
#include "Orient_teoric_square_rotZ.h"

#include "trajectory_rotXY.h"
#include "velocity_rotXY.h"
#include "Length_teoric_square_rotXY.h"
#include "Beta_teoric_square_rotXY.h"
#include "Pos_teoric_square_rotXY.h"
#include "Orient_teoric_square_rotXY.h"

#include <qwt_plot_renderer.h>
#include <fstream>
using namespace std;


//Serial No of the spatial phidgets
#define IDbeta1 122974 
//#define IDbeta1 165915
#define IDbeta2 114127
#define IDbeta3 114131

#define POS_MAX 75.0
#define POS_MIN 165.0
#define POS_HOME 120.0
#define SPEED_MIN 0.4
#define SPEED_MAX 6400   
#define NUM_SAMPLES 192 

typedef struct 
{ 
double targetPos;
double setVel;
} Tinfo;


std::string cont_name="Phidget1061";
std::string config_file="../src/xml/base_phidget1061_config.xml";

class CGOUGH_STEWART_MANAGER : public QDialog, private Ui::gough_stewart_manager_gui
{
  Q_OBJECT
  private: 
    QDialog *graph3d;
    QwtPlot *plots[21];
    //CPhidgetAdvancedServoHandle servo;
    CPHIDGET1061 *controller;
    CPHIDGET1049 beta_sensor[3];
    int IDbeta[3];//={IDbeta1, IDbeta2, IDbeta3}; 
    CEventServer *event_server;//=CEventServer::instance();
    std::list<std::string> events;
    int i,l,k,z;
    bool SensorsEnabled;
    const char* DeviceType;
    int max_servos;
    int SensorState[3];
    int channel;
    float p,P,deltha;
    char buf[100];
    std::vector<float> m,beta,endefector,rot,offset;
    std::vector<float> orient_angles;
    #ifndef _HAVE_XSD
    TMotorConfig config;
    #endif
std::string motor_ready;
std::vector<float> position,velocity,accel,current;
float Length_experimental[NumSamples][6];
float Beta_experimental[NumSamples][3]; 
float Pos_experimental[NumSamples][3];
float Orient_experimental[NumSamples][3];

float Length_teoric[NumSamples][6];
float Beta_teoric[NumSamples][3]; 
float Pos_teoric[NumSamples][3];
float Orient_teoric[NumSamples][3];
float nr[NumSamples][3];
float iter;
float error;
float ms;
	
float Pos_error[NumSamples][3];
float Orient_error[NumSamples][3];
std::vector<float> max_pos,min_pos;
std::vector<float> gear_ratio;
std::vector<bool> enable,relative;
std::vector<int> enc_res;
int event_id;//=-1;
std::vector<float> pos_slide, vel_slide;
  Tinfo SlidersInfo[8];
  protected: 
    void timerEvent(QTimerEvent * event); 
  public:
    CGOUGH_STEWART_MANAGER();	
    void UpdateInfo(void);
  public slots:
    void Up_slot(void);
    void Down_slot(void);
    void Home_slot(void);
    void Bars1_slot(void);
    void Bars2_slot(void);
    void CloseAll_slot(void);
    void Stop_slot(void);
    void ResetValues_slot(void);
    int Graf_slot(void);
    void ServoChannel_slot(int channel);
    void targetPosition_slot(int pos);
    void Velocity_slot(int vel);
    void Move_slot(void);
    void Stop_single_slot(void);
    void Line_slot(void);
    void Square_slot(void);
    void Rotx_slot(void);
    void Roty_slot(void);
    void Rotz_slot(void);
    void Rotxy_slot(void);
    void Circle_slot(void);
    void Helix_slot(void);
    void UseSensors_slot(int state);
};
 

#endif
