#include "operations.h"

float norm(const std::vector<float>& v)
{
  return(sqrt(pow(v[0],2.0)+pow(v[1],2.0)+pow(v[2],2.0)));
}

float norm6(const std::vector<float>& v)
{
  return(sqrt(pow(v[0],2.0)+pow(v[1],2.0)+pow(v[2],2.0)+pow(v[3],2.0)+pow(v[4],2.0)+pow(v[5],2.0)));
}

float abs(const float& num)
{
  float aux;
  if (num > 0) aux=num;
  else aux=-num;
  return(aux);
}

std::vector<float> cross(const std::vector<float>& v0,const std::vector<float>& v1)
{
  std::vector<float> cross;
  
  cross.push_back(v0[1]*v1[2]-v0[2]*v1[1]);
  cross.push_back(v0[2]*v1[0]-v0[0]*v1[2]);
  cross.push_back(v0[0]*v1[1]-v0[1]*v1[0]);

  return(cross);
}

std::vector<float> multiplyVectorMatrix( const std::vector<float>& v,const std::vector<float>& R)
{
unsigned int i=0;
unsigned int j=0;
std::vector<float> result;
float tmp=0.0;
for (i=0;i<3;i++)
{ 
 for (j=0;j<3;j++)
    tmp+=v[j]*R[j+3*i];
 result.push_back(tmp);
 tmp=0.0;
}
return(result); 
}

std::vector<float> multiplyVectorMatrix6( const std::vector<float>& v,const std::vector<float>& R)
{
unsigned int i=0;
unsigned int j=0;
std::vector<float> result;
float tmp=0.0;
for (i=0;i<6;i++)
{ 
 for (j=0;j<6;j++)
    tmp+=v[j]*R[i*6+j];
 result.push_back(tmp);
 tmp=0.0;
}
return(result); 
}

std::vector<float> multiplyVectorEscalar( const float k,const std::vector<float>& v)
{
unsigned int i=0;
std::vector<float> result;
for (i=0;i<3;i++)
 result.push_back(k*v[i]);
return(result); 
}

std::vector<float> transposeMatrix(const std::vector<float>& R)
{
unsigned int i=0;
unsigned int j=0;
std::vector<float> result;
float tmp=0.0;
for (i=0;i<3;i++)
 for (j=0;j<3;j++)
   result.push_back(R[i+3*j]);
return(result); 
}
std::vector<float> diffpoints(const std::vector<float>& p0,const std::vector<float>& p1)
{
  std::vector<float> diff;
  int                i;

  if (p0.size()!=3 | p1.size()!=3)  for (i=0;i<3;i++)  diff.push_back(0.0);
  else  for (i=0;i<3;i++)  diff.push_back(p1[i]-p0[i]);

  return(diff);
}

std::vector<float> diff6(const std::vector<float>& p0,const std::vector<float>& p1)
{
  std::vector<float> diff;
  int                i;

  if (p0.size()!=6 | p1.size()!=6)  for (i=0;i<6;i++)  diff.push_back(0.0);
  else  for (i=0;i<6;i++)  diff.push_back(p1[i]-p0[i]);

  return(diff);
}

std::vector<float> addVectors(const std::vector<float>& p0,const std::vector<float>& p1)
{
  std::vector<float> add;
  unsigned int i=0;
  for (i=0;i<3;i++)
  add.push_back(p0[i]+p1[i]); 
 return(add);
}

std::vector<float> addVectors6(const std::vector<float>& p0,const std::vector<float>& p1)
{
  std::vector<float> add;
  unsigned int i=0;
  for (i=0;i<6;i++)
  add.push_back(p0[i]+p1[i]); 
 return(add);
}

std::vector<float> computeEndefector(const std::vector<float>& p4,const std::vector<float>& p5,const std::vector<float>& p6,const float& p)
{
  std::vector<float> diff1,diff2,endefector,vectorproduct;
  float              norma;
  int                i;

  if (p4.size()!=3 | p5.size()!=3 | p6.size()!=3)
    for (i=0;i<3;i++)  endefector.push_back(0.0);
  else
  {
    diff1 = diffpoints(p4,p5);
    diff2 = diffpoints(p4,p6);
    norma = norm(diffpoints(diff1,diffpoints(p6,p4)));
    for (i=0;i<3;i++)
    {
      endefector.push_back(p4[i]+((sqrt(3.0)*p/3.0)*(diff1[i]+diff2[i])/norma));
    }
    //vectorproduct = cross(diff1,diff2);
    //for (i=0;i<3;i++)
    //{
    //  endefector.push_back(vectorproduct[i]/norm(vectorproduct));
   // }
  }
  return(endefector);
}

std::vector<float> computeRot(const std::vector<float>& p4,const std::vector<float>& p5,const std::vector<float>& p6)
{
  std::vector<float> u,v,w,rot,vectorproduct;
  float              norma;
  int                i;

  if (p4.size()!=3 | p5.size()!=3 | p6.size()!=3)
    for (i=0;i<9;i++) rot.push_back(0.0);
  else
  {
    u = diffpoints(p6,p5);
    norma = norm(u);
    for (i=0;i<3;i++)
      u[i]=u[i]/norma;
    v = addVectors(diffpoints(p4,p6),diffpoints(p4,p5));
    norma=norm(v);
    for (i=0;i<3;i++)
      v[i]=v[i]/norma;
    w=cross(u,v); 
    for (i=0;i<3;i++)
    rot.push_back(u[i]);
    for (i=0;i<3;i++)
    rot.push_back(v[i]);
    for (i=0;i<3;i++)
    rot.push_back(w[i]);
  }
 return(rot);
}

void shift(std::vector<float>& v)
{
 float tmp;
  tmp=v.front();
  v.erase(v.begin());
  v.push_back(tmp);
}

std::vector<float> computeJacobian(const float& P,const float& p,const float& deltha,const std::vector<float>& pos,const std::vector<float>& rot)
{
  std::vector<float> J;
  std::vector<float> p1,p2,p3,p4,p5,p6,q1,q2,q3,q4,q5,q6;
  std::vector<float> bar1,bar2,bar3,bar4,bar5,bar6;
  std::vector<float> r1,r2,r3,r4,r5,r6;
  std::vector<float> e1,e2,e3,e4,e5,e6;
  std::vector<float> u1,u2,u3,u4,u5,u6;
  int i,k;
  std::vector<float> p4aux,p5aux,p6aux,q4aux,q5aux,q6aux;
  
  p1.push_back(0.0);
  p1.push_back(0.0);
  p1.push_back(0.0);

  p2.push_back(P);
  p2.push_back(0.0);
  p2.push_back(0.0);

  p3.push_back(P/2.0);
  p3.push_back(sqrt(3)*P/2.0);
  p3.push_back(0.0);

  p4.push_back(0.0);
  p4.push_back(-sqrt(3.0)*p/3.0);
  p4.push_back(0.0);

  p5.push_back(p/2.0);
  p5.push_back(sqrt(3.0)*p/(2.0*3.0));
  p5.push_back(0);

  p6.push_back(-p/2.0)  ;
  p6.push_back(sqrt(3.0)*p/(2.0*3.0));
  p6.push_back(0.0);

  q1.push_back(deltha);
  q1.push_back(0.0);
  q1.push_back(0.0);

  q2.push_back(P-deltha/2.0);
  q2.push_back(deltha/2.0*sqrt(3.0));
  q2.push_back(0.0);

  q3.push_back(P/2.0-deltha/2.0);
  q3.push_back(P*sqrt(3.0)/2.0-deltha/2.0*sqrt(3.0));
  q3.push_back(0.0);

  q4.push_back(deltha/2.0);
  q4.push_back(-sqrt(3.0)*p/3.0+sqrt(3.0)*deltha/2.0);
  q4.push_back(0.0);

  q5.push_back(p/2.0-deltha);
  q5.push_back(sqrt(3.0)*p/(2.0*3.0));
  q5.push_back(0.0);

  q6.push_back(-p/2.0+deltha/2.0);
  q6.push_back(sqrt(3.0)*p/(2.0*3.0)-sqrt(3.0)*deltha/2.0);
  q6.push_back(0.0);
 
p4= addVectors(pos,multiplyVectorMatrix(p4,rot));
p5= addVectors(pos,multiplyVectorMatrix(p5,rot));
p6= addVectors(pos,multiplyVectorMatrix(p6,rot));
q4= addVectors(pos,multiplyVectorMatrix(q4,rot));
q5= addVectors(pos,multiplyVectorMatrix(q5,rot));
q6= addVectors(pos,multiplyVectorMatrix(q6,rot));
 
bar1 = diffpoints(q1,p4);
bar2 = diffpoints(p2,q4);
bar3 = diffpoints(q2,p5);
bar4 = diffpoints(p3,q5);
bar5 = diffpoints(q3,p6);
bar6 = diffpoints(p1,q6);
 
e1 = multiplyVectorEscalar(1/norm(bar1),bar1);
e2 = multiplyVectorEscalar(1/norm(bar2),bar2);
e3 = multiplyVectorEscalar(1/norm(bar3),bar3);
e4 = multiplyVectorEscalar(1/norm(bar4),bar4);
e5 = multiplyVectorEscalar(1/norm(bar5),bar5);
e6 = multiplyVectorEscalar(1/norm(bar6),bar6);
 
r1 = multiplyVectorEscalar(1/norm(diffpoints(pos,p4)),diffpoints(pos,p4));
r2 = multiplyVectorEscalar(1/norm(diffpoints(pos,q4)),diffpoints(pos,q4));
r3 = multiplyVectorEscalar(1/norm(diffpoints(pos,p5)),diffpoints(pos,p5));
r4 = multiplyVectorEscalar(1/norm(diffpoints(pos,q5)),diffpoints(pos,q5));
r5 = multiplyVectorEscalar(1/norm(diffpoints(pos,p6)),diffpoints(pos,p6));
r6 = multiplyVectorEscalar(1/norm(diffpoints(pos,q6)),diffpoints(pos,q6));

u1=cross(e1,r1);
u2=cross(e2,r2);
u3=cross(e3,r3);
u4=cross(e4,r4);
u5=cross(e5,r5);
u6=cross(e6,r6);

for(i=0;i<3;i++)
 J.push_back(e1[i]);
for(i=0;i<3;i++)
 J.push_back(u1[i]);

for(i=0;i<3;i++)
 J.push_back(e2[i]);
for(i=0;i<3;i++)
 J.push_back(u2[i]);

for(i=0;i<3;i++)
 J.push_back(e3[i]);
for(i=0;i<3;i++)
 J.push_back(u3[i]);

for(i=0;i<3;i++)
 J.push_back(e4[i]);
for(i=0;i<3;i++)
 J.push_back(u4[i]);

for(i=0;i<3;i++)
 J.push_back(e5[i]);
for(i=0;i<3;i++)
 J.push_back(u5[i]);

for(i=0;i<3;i++)
 J.push_back(e6[i]);
for(i=0;i<3;i++)
 J.push_back(u6[i]);

return(J);
}

void showvector(std::vector<float>& vector)
{
  int i;

  for(i=0;i<(int)vector.size();i++) std::cout<< vector[i] << "  ";
  std::cout<< std::endl;
}

void showMatrix(std::vector<float>& vector,int dim)
{
  int i;

  for(i=0;i<(int)vector.size();i++) { std::cout<< vector[i] << "  ";
  if(((i+1)%dim)==0) std::cout<< std::endl;}
  std::cout<< std::endl;
}

void matrix_inverse(float *Min, float *Mout, int actualsize) 
{
    /* Loop variables */
    int i, j, k;
    /* Sum variables */
    float sum,x,tmp;
    /*  Copy the input matrix to output matrix */
    for(i=0; i<(actualsize*actualsize); i++) { Mout[i]=Min[i];}
    /* Add small value to diagonal if diagonal is zero */
    for(i=0; i<actualsize; i++)
    {  
        j=i*actualsize+i;
        if((Mout[j]<1e-12)&&(Mout[j]>-1e-12)){ Mout[j]=1e-12; }
    }
    /* Matrix size must be larger than one */
    if (actualsize <= 1) return;
    
    for (i=1; i < actualsize; i++) {
        Mout[i] /= Mout[0]; /* normalize row 0 */
    }
    
    for (i=1; i < actualsize; i++)  {
        for (j=i; j < actualsize; j++)  { /* do a column of L */
            sum = 0.0;
            for (k = 0; k < i; k++) {
                sum += Mout[j*actualsize+k] * Mout[k*actualsize+i];
            }
            Mout[j*actualsize+i] -= sum;
        }
        if (i == actualsize-1) continue;
        for (j=i+1; j < actualsize; j++)  {  /* do a row of U */
            sum = 0.0;
            for (k = 0; k < i; k++) {
                sum += Mout[i*actualsize+k]*Mout[k*actualsize+j];
            }
            Mout[i*actualsize+j] = (Mout[i*actualsize+j]-sum) / Mout[i*actualsize+i];
        }
    }
    for ( i = 0; i < actualsize; i++ )  /* invert L */ {
        for ( j = i; j < actualsize; j++ )  {
            x = 1.0;
            if ( i != j ) {
                x = 0.0;
                for ( k = i; k < j; k++ ) {
                    x -= Mout[j*actualsize+k]*Mout[k*actualsize+i];
                }
            }
            Mout[j*actualsize+i] = x / Mout[j*actualsize+j];
        }
    }
    for ( i = 0; i < actualsize; i++ ) /* invert U */ {
        for ( j = i; j < actualsize; j++ )  {
            if ( i == j ) continue;
            sum = 0.0;
            for ( k = i; k < j; k++ ) {
                sum += Mout[k*actualsize+j]*( (i==k) ? 1.0 : Mout[i*actualsize+k] );
            }
            Mout[i*actualsize+j] = -sum;
        }
    }
    for ( i = 0; i < actualsize; i++ ) /* final inversion */ {
        for ( j = 0; j < actualsize; j++ )  {
            sum = 0.0;
            for ( k = ((i>j)?i:j); k < actualsize; k++ ) {
                sum += ((j==k)?1.0:Mout[j*actualsize+k])*Mout[k*actualsize+i];
            }
            Mout[j*actualsize+i] = sum;
        }
    }
}

std::vector<float> matrixr3d(float wx, float wy, float wz)
{  
std::vector<float> R;
R.push_back(cos(wz)*cos(wy));
R.push_back(cos(wz)*sin(wy)*sin(wx)-sin(wz)*cos(wx));
R.push_back(cos(wz)*sin(wy)*cos(wx)+sin(wz)*sin(wx) );
R.push_back(sin(wz)*cos(wy));
R.push_back(sin(wz)*sin(wy)*sin(wx)+cos(wz)*cos(wx));
R.push_back(sin(wz)*sin(wy)*cos(wx)-cos(wz)*sin(wx));
R.push_back(-sin(wy));
R.push_back(cos(wy)*sin(wx));
R.push_back(cos(wx)*cos(wy));
return(R);                              
}

