#include "phidget1049.h"
#include <iostream>
#include <sstream>
#include <math.h>

CPHIDGET1049::CPHIDGET1049(void)
{
  //Init the spatial handle
  this->spatial= 0;
  this->attached=false;
  //create the advanced servo object
  CPhidgetSpatial_create(&this->spatial);
  //Set the handlers to be run when the device is plugged in or opened from software, unplugged or closed from software, or generates an error.
  CPhidget_set_OnAttach_Handler((CPhidgetHandle)this->spatial, AttachHandler, NULL);
  CPhidget_set_OnDetach_Handler((CPhidgetHandle)this->spatial, DetachHandler, NULL);
  CPhidget_set_OnError_Handler((CPhidgetHandle)this->spatial, ErrorHandler, NULL);
  //Registers a callback that will run according to the set data rate that will return the spatial data changes
  //Requires the handle for the Spatial, the callback handler function that will be called, 
  //and an arbitrary pointer that will be supplied to the callback function (may be NULL)
  CPhidgetSpatial_set_OnSpatialData_Handler(this->spatial, SpatialDataHandler, &this->spatial);
}


int CPHIDGET1049::AttachHandler(CPhidgetHandle spatial, void *userptr)
{
	int serialNo;
	CPhidget_getSerialNumber(spatial, &serialNo);
	printf("Spatial %10d attached!", serialNo);
        
	return 0;
}

int CPHIDGET1049::DetachHandler(CPhidgetHandle spatial, void *userptr)
{
	int serialNo;
	CPhidget_getSerialNumber(spatial, &serialNo);
	printf("Spatial %10d detached! \n", serialNo);

	return 0;
}


int CPHIDGET1049::ErrorHandler(CPhidgetHandle spatial, void *userptr, int ErrorCode, const char *unknown)
{
	printf("Error handled. %d - %s \n", ErrorCode, unknown);
	return 0;
}


int CPHIDGET1049::SpatialDataHandler(CPhidgetSpatialHandle spatial, void *userptr, CPhidgetSpatial_SpatialEventDataHandle *data, int count)
{
	CPHIDGET1049 *spatial_phid = (CPHIDGET1049 *) userptr;
	int i;
        double modul;
        double Gx,Gy,Gz;
	//printf("Number of Data Packets in this event: %d\n", count);
	
	for(i = 0; i < count; i++)
	{
        spatial_phid->Gx=data[i]->acceleration[0];
        spatial_phid->Gy=data[i]->acceleration[1];
        spatial_phid->Gz=data[i]->acceleration[2];
//  	printf("=== Data Set: %d ===\n", i);
//	printf("Acceleration> x: %5f  y: %5f  z: %5f\n",spatial_phid->Gx,spatial_phid->Gy,spatial_phid->Gz);
//	printf("Timestamp> seconds: %d -- microseconds: %d\n", data[i]->timestamp.seconds, data[i]->timestamp.microseconds);
        modul = sqrt (data[i]->acceleration[0]*data[i]->acceleration[0]+data[i]->acceleration[1]*data[i]->acceleration[1]+data[i]->acceleration[2]*data[i]->acceleration[2]);        	
	spatial_phid->angle = asin(-spatial_phid->Gy/modul);//in radians
        //printf("Modul of gravity: %5f  and angle: %5f\n",modul,spatial_phid->angle); 
	}
//	printf("---------------------------------------------\n");
	return 0;
}

void CPHIDGET1049::open(int serialNo)
{
     //open the spatial object for device connections
     CPhidget_open((CPhidgetHandle)this->spatial, serialNo);
}

int CPHIDGET1049::wait (int milliseconds)
{
  int result;
  const char *err;
	//get the program to wait for an advanced servo device to be attached
	printf("Waiting for Spatial Phidget to be attached....\n\r");
	if((result = CPhidget_waitForAttachment((CPhidgetHandle)this->spatial, milliseconds)))
	{
		CPhidget_getErrorDescription(result, &err);
		printf("Problem waiting for attachment: %s\n", err);
		printf("Check the Spatial Phidget connection\n");
	        this->attached=false;
		return 0;

 	 }
        this->attached=true;
	return 1;
}

int CPHIDGET1049::displayProperties(void)
{
    CPhidget_getDeviceType((CPhidgetHandle )this->spatial, &this->DeviceType);
    CPhidget_getSerialNumber((CPhidgetHandle )this->spatial, &this->serialNo);
    CPhidget_getDeviceVersion((CPhidgetHandle )this->spatial, &this->version);
    CPhidgetSpatial_getAccelerationAxisCount((CPhidgetSpatialHandle)this->spatial, &this->numAccelAxes);
    CPhidgetSpatial_getGyroAxisCount((CPhidgetSpatialHandle)this->spatial, &this->numGyroAxes);
    CPhidgetSpatial_getCompassAxisCount((CPhidgetSpatialHandle)this->spatial, &this->numCompassAxes);
    CPhidgetSpatial_getDataRateMax((CPhidgetSpatialHandle)this->spatial, &this->dataRateMax);
    CPhidgetSpatial_getDataRateMin((CPhidgetSpatialHandle)this->spatial, &this->dataRateMin);
    printf("%s\n", this->DeviceType);
    printf("Serial Number: %10d\nVersion: %8d\n", serialNo, version);
    printf("Number of Accel Axes: %i\n", numAccelAxes);
    printf("Number of Gyro Axes: %i\n", numGyroAxes);
    printf("Number of Compass Axes: %i\n", numCompassAxes);
    printf("datarate> Max: %d  Min: %d\n", dataRateMax, dataRateMin);
    return 0;

}

void CPHIDGET1049::setDataRate(int ms)
{
	//Set the data rate for the spatial events
	CPhidgetSpatial_setDataRate(this->spatial, ms);
}

void CPHIDGET1049::getAcc(void)
{
CPhidgetSpatial_getAcceleration(spatial, 0,&this->Gx);
CPhidgetSpatial_getAcceleration(spatial, 1,&this->Gy);
CPhidgetSpatial_getAcceleration(spatial, 2,&this->Gz);
}

void CPHIDGET1049::getAngle(void)
{
double modul;
this->getAcc();
modul = sqrt(this->Gx*this->Gx +this->Gy*this->Gy+this->Gz*this->Gz);
this->angle = asin(-this->Gy);//in radians
}

void CPHIDGET1049::close(void)
{
CPhidget_close((CPhidgetHandle)this->spatial);
}

CPHIDGET1049::~CPHIDGET1049()
{
CPhidget_delete((CPhidgetHandle)this->spatial);
}


