#include "phidget1061.h"
#include "eventexceptions.h"
#include <iostream>
#include <sstream>
#ifdef _HAVE_XSD
#include "xml/phidget1061_cfg_file.hxx"
#endif 

CPHIDGET1061::CPHIDGET1061(std::string& cont_id):CMotorControl(cont_id,6)
{
  this->LowerPosLim=LPL;
  this->UpperPosLim=UPL;
  this->PosRes=POS_RESOLUTION;
  this->VelRes=VEL_RESOLUTION;
  this->UpperVelLim=UVL;
  this->AccRes=ACC_RESOLUTION;
  this->UpperAccLim=UAL;
  //Init the advanced servo handle
  this->servo = 0;
  this->attached=false;
  //create the advanced servo object  
  CPhidgetAdvancedServo_create(&this->servo);
  //this->displayProperties();
  //CPhidgetAdvancedServo_set_OnPositionChange_Handler((CPhidgetAdvancedServoHandle)this->servo, PositionChangeHandler, NULL);
  //CPhidgetAdvancedServo_set_OnCurrentChange_Handler((CPhidgetAdvancedServoHandle)this->servo, CurrentChangeHandler, NULL);
}


int CPHIDGET1061::AttachHandler(CPhidgetHandle SERVO, void *userptr)
{
  int serialNo;
  const char *name;  
  if( CPhidget_getDeviceName (SERVO, &name)!=EPHIDGET_OK)std::cout << "Error: Get Device Name" <<std::endl;
  if( CPhidget_getSerialNumber(SERVO, &serialNo)!=EPHIDGET_OK)std::cout << "Error: Get Serial Number" <<std::endl;
  printf("%s %10d attached!\n", name, serialNo);
  return 0;
}

int CPHIDGET1061::DetachHandler(CPhidgetHandle SERVO, void *userptr)
{
  int serialNo;
  const char *name;
  if( CPhidget_getDeviceName (SERVO, &name)!=EPHIDGET_OK)std::cout << "Error: Get Device Name" <<std::endl;
  if( CPhidget_getSerialNumber(SERVO, &serialNo)!=EPHIDGET_OK)std::cout << "Error: Get Serial Number" <<std::endl;
  printf("%s %10d detached!\n", name, serialNo);
  return 0;
}

int CPHIDGET1061::ErrorHandler(CPhidgetHandle SERVO, void *userptr, int ErrorCode, const char *Description)
{
  printf("Error handled. %d - %s\n", ErrorCode, Description);
  return 0;
}

int CPHIDGET1061::PositionChangeHandler(CPhidgetAdvancedServoHandle ADVSERVO, void *usrptr, int Index, double Value)
{
  //printf("Motor: %d > Current Position: %f\n", Index, Value);
  return 0;
}

int CPHIDGET1061::VelocityChangeHandler(CPhidgetAdvancedServoHandle ADVSERVO, void *usrptr, int Index, double Value)
{
//  printf("Motor: %d > Current Velocity: %f\n", Index, Value);
  return 0;
}

int CPHIDGET1061::CurrentChangeHandler(CPhidgetAdvancedServoHandle ADVSERVO, void *usrptr, int Index, double Value)
{
  //printf("Motor: %d > Current current: %f\n", Index, Value);
  return 0;
}

void CPHIDGET1061::open(void)
{
  int i=0;
  this->phidget_access.enter(); 
  try{
   // Opening phidget
    if(CPhidget_open((CPhidgetHandle)this->servo, -1)!=EPHIDGET_OK) std::cout << "Error: Open Phidget" <<std::endl;
  }catch(CException &e){
    /* handle exceptions */
    std::cout << e.what() << std::endl;
    throw;
    this->phidget_access.exit();
  }
  this->phidget_access.exit();
}

int CPHIDGET1061::wait(int milliseconds)
{
  int i=0,result;
  const char *err;
  //get the program to wait for an advanced servo device to be attached
  try
  {  
    printf("Waiting for Servo Phidget to be attached....\n\r");
    if((result = CPhidget_waitForAttachment((CPhidgetHandle)this->servo, milliseconds)))
    {
      CPhidget_getErrorDescription(result, &err);
      printf("Problem waiting for attachment: %s\n", err);
      printf("Check the Servo Controller Phidget connection\n");
      this->attached=false;
      return 0;
    }
   this->attached=true;
  //If the servo is not engaged, then the position cannot be read
  for(i=0;i<6;i++)
  {
  if(CPhidgetAdvancedServo_setServoParameters(this->servo,i,1065.0,1985.0,255.0,50.0)!=	EPHIDGET_OK) std::cout << "Wait: Error in set servo parameters" << std::endl; ; 
  if(CPhidgetAdvancedServo_setSpeedRampingOn(this->servo,i,1)!=EPHIDGET_OK) std::cout << "Wait: Error in Speed Ramping On" << std::endl;
  if(CPhidgetAdvancedServo_setEngaged(this->servo,i,1)!=EPHIDGET_OK) std::cout << "Wait: Error in set Engaged" << std::endl;
  }
  return 1;
  }catch(CException& e){
   std::cout << e.what() << std::endl;
   throw;
  }
}

int CPHIDGET1061::displayProperties(void)
{
  try{
    this->phidget_access.enter();
    //Display phidget features
    if(CPhidget_getDeviceType((CPhidgetHandle)this->servo, &this->DeviceType)!=EPHIDGET_OK) std::cout << "Error in get Device Type" << std::endl;
    if(CPhidget_getSerialNumber((CPhidgetHandle)this->servo, &this->serialNo)!=EPHIDGET_OK) std::cout << "Error in get Serial Number" << std::endl;
    if(CPhidget_getDeviceVersion((CPhidgetHandle)this->servo, &this->version)!=EPHIDGET_OK) std::cout << "Error in get Device Version" << std::endl;
    if(CPhidgetAdvancedServo_getMotorCount (this->servo, &this->numMotors)!=EPHIDGET_OK) std::cout << "Error in get Motor Count" << std::endl;
    this->phidget_access.exit();
    printf("%s\n", this->DeviceType);
    printf("Serial Number: %10d\nVersion: %8d\n# Motors: %d\n", this->serialNo, this->version, this->numMotors);
    return 0;
  }catch(CException &e){
    // handle exceptions
    std::cout << e.what() << std::endl; 
    throw;}
}

void CPHIDGET1061::getNumMotors (int *numMotors)
{
  this->phidget_access.enter();
  if(CPhidgetAdvancedServo_getMotorCount (this->servo,numMotors)!=EPHIDGET_OK) std::cout << "Error in get  number of motors" << std::endl;
  this->phidget_access.exit();
}

int CPHIDGET1061::getDeviceType(const char **deviceType)
{
  this->phidget_access.enter();
  if(CPhidget_getDeviceType((CPhidgetHandle)this->servo, &this->DeviceType)!=EPHIDGET_OK) std::cout << "Error in get Device Type" << std::endl;
  this->phidget_access.exit();
}

int CPHIDGET1061::getSerialNumber(int serialNo)
{
  this->phidget_access.enter();
  if(CPhidget_getSerialNumber((CPhidgetHandle)this->servo, &this->serialNo)!=EPHIDGET_OK)std::cout << "Error in get Serial Number" << std::endl;
  this->phidget_access.exit();
}

int CPHIDGET1061::getDeviceVersion(int *version)
{
  this->phidget_access.enter();
  if(CPhidget_getDeviceVersion((CPhidgetHandle)this->servo, &this->version)!=EPHIDGET_OK) std::cout << "Error in get Device Version" << std::endl;
  this->phidget_access.exit();
}

void CPHIDGET1061::getServoType( int index, CPhidget_ServoType *servoType)
{
  this->phidget_access.enter();
  if(CPhidgetAdvancedServo_getServoType(this->servo, index, servoType)!=EPHIDGET_OK) std::cout << "Error in get Servo Type" << std::endl;  
  std::cout << "Servo type[" << index << "] is: " << *servoType << std::endl;
  this->phidget_access.exit();
}

void CPHIDGET1061::setServoType( int index, CPhidget_ServoType servoType)
{
int code=0; 
 this->phidget_access.enter();
 code =CPhidgetAdvancedServo_setServoType(this->servo, index, servoType);
  if(code!=EPHIDGET_OK)std::cout << "Error: " << code << " in set Servo Type" << std::endl;
  this->phidget_access.exit();
}

int CPHIDGET1061::setServoParameters( int index, double min_us,double max_us,double degrees,double velocity_max)
{
  int code=0;
  this->phidget_access.enter();
  code =CPhidgetAdvancedServo_setServoParameters(this->servo,index,min_us,max_us,degrees,velocity_max);
  if (code !=EPHIDGET_OK) std::cout << "Error: " << code << " in set Servo Parameters" << std::endl;
  this->phidget_access.exit();
}

void CPHIDGET1061::angles_to_controller(std::vector<float>& angles,std::vector<float>& counts)
{
int num=this->get_num_axis();
int i=0;
if(angles.size()!=this->get_num_axis())
  {
    /* handle exceptions */
  }
  else
  { 
    counts.clear();
    counts.resize(this->get_num_axis());
    if(this->encoder_resolution==-1)
    {
      /* handle exceptions */
    std::cout << "else" << std::endl;
    }
    else
    {
      if(this->gear_factor==-1.0)
      {
        /* handle exceptions */
      }
      else
        {
        for(i=0; i< num;i++)
        counts[i]=(float)((angles[i]-(this->LowerPosLim))/(this->PosRes));
        }
    }
  }

}

void CPHIDGET1061::speeds_to_controller(std::vector<float>& speeds,std::vector<float>& counts)
{
 int num=this->get_num_axis();
 int i=0;
  if(speeds.size()!=this->get_num_axis())
  {
    /* handle exceptions */
  }
  else
  {
    counts.clear();
    counts.resize(this->get_num_axis());
    if(this->gear_factor==-1.0)
    {
      /* handle exceptions */
    }
    else
    { 
      for(i=0; i< num;i++)
      counts[i]=(float)(speeds[i]/this->VelRes);
    }  
  }
}

void CPHIDGET1061::accels_to_controller(std::vector<float>& accels,std::vector<float>& counts)
{
  int num=this->get_num_axis();
  int i=0;
  if(accels.size()!=this->get_num_axis())
  {
    /* handle exceptions */
  }
  else
  {
    counts.clear();
    counts.resize(this->get_num_axis());
    if(this->gear_factor==-1.0)
    {
      /* handle exceptions */
    }
    else
    {
      for(i=0; i< num;i++)
      counts[i]=(float)(accels[i]/this->AccRes); 
    }  
  }
}

void CPHIDGET1061::controller_to_angles(std::vector<float>& counts,std::vector<float>& angles)
{
  int num=this->get_num_axis();
  int i=0;
  if(counts.size()!=this->get_num_axis())
  {
    /* handle exceptions */
  }
  else
  {
    angles.clear();
    angles.resize(this->get_num_axis());
    if(this->encoder_resolution==-1)
    {
      /* handle exceptions */
    }
    else
    {
      if(this->gear_factor==-1.0)
      {
        /* handle exceptions */
      }
      else
      {  
        for(i=0; i< num;i++)
        angles[i]=((float)counts[i])*this->PosRes+this->LowerPosLim;
      }
    }
  }
}

void CPHIDGET1061::controller_to_speeds(std::vector<float>& counts,std::vector<float>& speeds)
{
  int num=this->get_num_axis();
  int i=0;
  if(counts.size()!=this->get_num_axis())
  {
    /* handle exceptions */
  }
  else
  {
    speeds.clear();
    speeds.resize(this->get_num_axis());
    if(this->gear_factor==-1.0)
    {
      /* handle exceptions */
    }
    else
    {
      for(i=0; i<num;i++)
      speeds[i]=((float)counts[i])*this->VelRes;
    }
  }
}

void CPHIDGET1061::controller_to_accels(std::vector<float>& counts,std::vector<float>& accels)
{
  int num=this->get_num_axis();
  int i=0;
  if(counts.size()!=this->get_num_axis())
  {
    /* handle exceptions */
  }
  else
  {
    accels.clear();
    accels.resize(this->get_num_axis());
    if(this->gear_factor==-1.0)
    {
      /* handle exceptions */
    }
    else
    {
      for(i=0; i<num;i++)      
      accels[i]=((float)counts[i])*this->AccRes;
    }
  }
}

void CPHIDGET1061::cont_set_accel_range(std::vector<float>& min, std::vector<float>& max)
{
}

void CPHIDGET1061::cont_set_velocity_range(std::vector<float>& min, std::vector<float>& max)
{

}

void CPHIDGET1061::cont_set_position_range(std::vector<float>& min, std::vector<float>& max)
{
 int num=this->get_num_axis();
  int i=0;
  std::vector <float> min_pos_angles,max_pos_angles;
  int code=0;
  this->phidget_access.enter();
  for(i=0;i<num;i++)
  {
    try
      {
      this->controller_to_angles(min,min_pos_angles);
      this->controller_to_angles(max,max_pos_angles);
    //std::cout << "Set "<< min[i] << " " << min_pos_angles[i] << " " << max[i] << " "<<max_pos_angles[i]  <<std::endl;
      code =CPhidgetAdvancedServo_setPositionMax (this->servo, i,max_pos_angles[i]);
      if(code!=EPHIDGET_OK)std::cout << "Error: " << code << " in set Position Max" << std::endl;
      code = CPhidgetAdvancedServo_setPositionMin (this->servo, i,min_pos_angles[i]);
      if(code!=EPHIDGET_OK)std::cout << "Error: " << code << " in set Position Min" << std::endl;
      }catch(CException& e){
      // handle exceptions 
        this->phidget_access.exit();
        std::cout << e.what() << std::endl;
        throw;
        }
  }
  this->phidget_access.exit();
}

void CPHIDGET1061::cont_set_encoder_resolution(std::vector<int>& enc_res)
{

}


void CPHIDGET1061::cont_set_gear_factor(std::vector<float>& gear)
{
  this->gear_factor=gear[0];
}

void CPHIDGET1061::cont_get_accel_range(std::vector<float>& min, std::vector<float>& max)
{
  int num=this->get_num_axis();
  int i=0;
  double accMax=0.0;
  double accMin=0.0;

  this->phidget_access.enter();
  for(i=0;i<num;i++)
  {
    try{
      if(CPhidgetAdvancedServo_getAccelerationMax (this->servo, i,&accMax)!=EPHIDGET_OK)std::cout << "Error in get Acceleration Max" << std::endl;
      if(CPhidgetAdvancedServo_getAccelerationMin (this->servo, i,&accMin)!=EPHIDGET_OK) std::cout << "Error in get Acceleration Min" << std::endl;
      min.push_back(accMin);
      max.push_back(accMax);
      std::cout << accMax << accMin << std::endl;
     }catch(CException& e){
      // handle exceptions
      this->phidget_access.exit();
      std::cout << e.what() << std::endl;
      throw;
     }
  }
  this->phidget_access.exit();
}

void CPHIDGET1061::cont_get_velocity_range(std::vector<float>& min, std::vector<float>& max)
{
  int num=this->get_num_axis();
  int i=0,index;
  double velMax=0.0;
  double velMin=0.0;

  this->phidget_access.enter();
  for(i=0;i<num;i++)
  {
    try{
      if(CPhidgetAdvancedServo_getVelocityMax (this->servo, i,&velMax)!=EPHIDGET_OK) std::cout << "Error in get Velocity Max" << std::endl;
      if(CPhidgetAdvancedServo_getVelocityMin (this->servo, i,&velMin)!=EPHIDGET_OK) std::cout << "Error in get Velocity Min" << std::endl;
      min.push_back(velMin);
      max.push_back(velMax);
      std::cout << velMax  << velMin << std::endl;
    }catch(CException& e){
      // handle exceptions 
      this->phidget_access.exit();
      std::cout << e.what() << std::endl;
      throw;
    }
  } 
  this->phidget_access.exit();
}

void CPHIDGET1061::cont_config_velocity_feedback(feedback_mode mode, float rate)
{
  this->get_velocity();
}

void CPHIDGET1061::cont_get_position_range(std::vector<float>& min, std::vector<float>& max)
{
  int num=this->get_num_axis();
  int i=0,index;
  double posMax, posMin;
  std::vector<float> posMax_counts,posMin_counts;

  this->phidget_access.enter();
  for(i=0;i<num;i++)
  {
    try{
      if(CPhidgetAdvancedServo_getPositionMax (this->servo, i,&posMax)!=EPHIDGET_OK) std::cout << "Error in get Position Max" << std::endl;
      if(CPhidgetAdvancedServo_getPositionMin (this->servo, i,&posMin)!=EPHIDGET_OK) std::cout << "Error in get Position Min" << std::endl;
      posMin_counts.push_back(posMin);
      posMax_counts.push_back(posMax);
       }catch(CException& e){
      /* handle exceptions */ 
       this->phidget_access.exit(); 
       std::cout << e.what() << std::endl;
       throw;
    }
  } 
  this->angles_to_controller(posMin_counts,min);
  this->angles_to_controller(posMax_counts,max);
  this->phidget_access.exit();
 // std::cout << "Max position: "<< posMax << " Min position: " << posMin << std::endl;
}

void CPHIDGET1061::cont_get_encoder_resolution(std::vector<int>& enc_res)
{

}

std::vector<float> CPHIDGET1061::cont_get_position(void)
{
  std::vector<float> position;
  int num=this->get_num_axis();
  int i=0,index,status,return_code;
  double pos=0;

  this->phidget_access.enter();
  position.clear();
  try
   {
     for(i=0;i<num;i++)
     {
      return_code = CPhidgetAdvancedServo_getPosition (this->servo,i,&pos); 
     // pos= (float)((int)(pos*100))/100;   //Limit the number of decimals in 3
      if(return_code !=EPHIDGET_OK)  std::cout << "Error in get Position. Error code: " << return_code <<std::endl;
      //std::cout << "Actual postition[" << i << "] "<< pos << std::endl;}        
      position.push_back((pos-this->LowerPosLim)/this->PosRes); 
     }

   }catch(CException& e){
    this->phidget_access.exit();
    std::cout << e.what() << std::endl;
    throw;
    }
 this->phidget_access.exit();
return position;
}

void CPHIDGET1061::cont_config_position_feedback(feedback_mode mode, float rate)
{
  this->get_position();
}

std::vector<float> CPHIDGET1061::cont_get_velocity(void)
{
  std::vector<float> velocity;
  int num=this->get_num_axis();
  int i=0,index;
  double vel=0;
  this->phidget_access.enter();
  velocity.clear();
  try
    {
      for(i=0;i<num;i++)
      {
        if(CPhidgetAdvancedServo_getVelocity (this->servo,i,&vel)!=EPHIDGET_OK) std::cout << "Error in get Velocity" << std::endl;
      //  vel= (float)((int)(vel*100))/100;   //Limit the number of decimals in 3
        velocity.push_back(vel/this->VelRes); 
      //  std::cout << velocity[i] << std::endl;
      }
    }catch(CException& e){
    this->phidget_access.exit();
    std::cout << e.what() << std::endl;
    throw;
    }
  this->phidget_access.exit();
return velocity;
}



std::vector<float> CPHIDGET1061::get_current(void)
{

  std::vector<float> current;
  int num=this->get_num_axis();
  int i=0,index,status,return_code;
  double curr=0;

  this->phidget_access.enter();
  current.clear();
  try
   {
     for(i=0;i<num;i++)
     {
      return_code = CPhidgetAdvancedServo_getCurrent (this->servo,i,&curr); 
      if(return_code !=EPHIDGET_OK)  std::cout << "Error in get Current. Error code: " << return_code <<std::endl;       
      current.push_back(curr*1000); //En mA 
     }

   }catch(CException& e){
    this->phidget_access.exit();
    std::cout << e.what() << std::endl;
    throw;
    }
 this->phidget_access.exit();
return current;
}

std::vector<float>  CPHIDGET1061::cont_get_limits(void)
{

}


void CPHIDGET1061::cont_set_gpio(int gpio_id,gpio_type type)
{
  
}

void CPHIDGET1061::cont_write_gpio(int gpio_id,bool value)
{

}

void CPHIDGET1061::cont_write_gpio(int gpio_id,float value)
{
  /* do nothing */
}

float CPHIDGET1061::cont_read_gpio(int gpio_id)
{

}


void CPHIDGET1061::cont_config(void *config)
{
  //Set the handlers to be run when the device is plugged in or opened from software, unplugged or closed from software, or generates an error.
  if(CPhidget_set_OnAttach_Handler((CPhidgetHandle ) this->servo, AttachHandler, NULL)!=EPHIDGET_OK)std::cout << " Error in Attach Handler" <<std::endl;
  if(CPhidget_set_OnDetach_Handler((CPhidgetHandle ) this->servo, DetachHandler, NULL)!=EPHIDGET_OK) std::cout << "Error in Detach Handler" <<std::endl;
  if(CPhidget_set_OnError_Handler((CPhidgetHandle ) this->servo, ErrorHandler, NULL)!=EPHIDGET_OK)std::cout << "Error in Error Handler" <<std::endl;
  //Enable Logging messages
  if(CPhidget_enableLogging(PHIDGET_LOG_CRITICAL,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging CRITICAL" <<std::endl;
  if(CPhidget_enableLogging(PHIDGET_LOG_ERROR,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging ERROR" <<std::endl;
  if(CPhidget_enableLogging(PHIDGET_LOG_WARNING,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging WARNING" <<std::endl;
  if(CPhidget_enableLogging(PHIDGET_LOG_DEBUG,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging DEBUG" <<std::endl;
  if(CPhidget_enableLogging(PHIDGET_LOG_INFO,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging INFO" <<std::endl;
  if(CPhidget_enableLogging(PHIDGET_LOG_VERBOSE,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging VERBOSE" <<std::endl;
}

void CPHIDGET1061::cont_config_limits_feedback(feedback_mode mode, float rate)
{
  
}

#ifdef _HAVE_XSD
void CPHIDGET1061::cont_save_config(std::string &filename)
{/*
  xml_schema::namespace_infomap map;
  float max_voltage, min_voltage;
  unsigned char cw_margin, ccw_margin;
  unsigned char cw_slope, ccw_slope;

  try{
    std::ofstream output_file(filename.c_str(),std::ios_base::out);
    this->get_voltage_limits(&min_voltage,&max_voltage);
    this->get_compliance_margin(&cw_margin,&ccw_margin);
    this->get_compliance_slope(&cw_slope,&ccw_slope);
    dynamixel_motor_config_t dyn_cfg(this->get_temperature_limit(),
                                     max_voltage,min_voltage,
                                     cw_margin,ccw_margin,
                                     cw_slope,ccw_slope,
                                     this->get_punch());
    map[""].name="";
    map[""].schema="dynamixel_motor_config.xsd";
    dynamixel_motor_config(output_file,dyn_cfg,map);
  }catch (const xml_schema::exception& e){
    std::ostringstream os;
    os << e;
    // handle exceptions 
    CMotorConfigException(_HERE_,os.str());
  }*/
}

void CPHIDGET1061::cont_load_config(std::string &filename)
{
   // try to open the specified file
  try{
    std::auto_ptr<phidget1061_config_t> cfg(phidget1061_config(filename.c_str(),xml_schema::flags::dont_validate));
  //Set the handlers to be run when the device is plugged in or opened from software, unplugged or closed from software, or generates an error.
  if(CPhidget_set_OnAttach_Handler((CPhidgetHandle ) this->servo, AttachHandler, NULL)!=EPHIDGET_OK)std::cout << " Error in Attach Handler" <<std::endl;
  if(CPhidget_set_OnDetach_Handler((CPhidgetHandle ) this->servo, DetachHandler, NULL)!=EPHIDGET_OK) std::cout << "Error in Detach Handler" <<std::endl;
  if(CPhidget_set_OnError_Handler((CPhidgetHandle ) this->servo, ErrorHandler, NULL)!=EPHIDGET_OK)std::cout << "Error in Error Handler" <<std::endl;
  //Enable Logging messages
  if(CPhidget_enableLogging(PHIDGET_LOG_CRITICAL,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging CRITICAL" <<std::endl;
  if(CPhidget_enableLogging(PHIDGET_LOG_ERROR,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging ERROR" <<std::endl;
  if(CPhidget_enableLogging(PHIDGET_LOG_WARNING,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging WARNING" <<std::endl;
  if(CPhidget_enableLogging(PHIDGET_LOG_DEBUG,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging DEBUG" <<std::endl;
  if(CPhidget_enableLogging(PHIDGET_LOG_INFO,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging INFO" <<std::endl;
  if(CPhidget_enableLogging(PHIDGET_LOG_VERBOSE,0)!=EPHIDGET_OK)std::cout << "Error in enable Logging VERBOSE" <<std::endl;

  }catch (const xml_schema::exception& e){
    std::cout << e << std::endl;
    /* handle exceptions */
    CMotorConfigException(_HERE_,"Impossible to load the configuration from a file");
  }
}
#endif

void CPHIDGET1061::cont_enable(std::vector<bool> &enable)
{
  int num=this->get_num_axis();
  int i=0;
  this->phidget_access.enter();
  for(i=0;i<num;i++)
  {
     if(CPhidgetAdvancedServo_setEngaged(this->servo,i,1)!=EPHIDGET_OK) std::cout << "Enable: Error in set Engaged" << std::endl;
  }
  this->phidget_access.exit();

}

void CPHIDGET1061::cont_disable(std::vector<bool> &disable)
{
  int num=this->get_num_axis();
  int i=0;

  this->phidget_access.enter();
  for(i=0;i<num;i++)
  {
     if(CPhidgetAdvancedServo_setEngaged(this->servo,i,0)!=EPHIDGET_OK) std::cout << "Disable: Error in set Engaged" << std::endl;
  }
  this->phidget_access.exit();
}

void CPHIDGET1061::cont_home(void)
{

}

void CPHIDGET1061::cont_load(std::vector<float>& position, std::vector<float>& velocity, std::vector<float>& accel)
{

  int i,return_code,status;
  int num=this->get_num_axis();
  double pos=0.0;
  double vel=0.0;
  double acc=0.0;
  try
  {
    this->phidget_access.enter();
    for(i=0;i<num;i++)
    {
      vel = ((float) velocity[i])*this->VelRes;
      return_code =CPhidgetAdvancedServo_setVelocityLimit (this->servo, i,vel);
      if(return_code!=EPHIDGET_OK) std::cout << "Load: Error setting velocity[" << i <<"] Error code: " << return_code << std::endl;
      acc = ((float) accel[i])*this->AccRes;
      return_code = CPhidgetAdvancedServo_setAcceleration (this->servo, i,acc);
      if(return_code!=EPHIDGET_OK) std::cout << "Load: Error setting acceleration[" << i <<"]: "<< acc <<" Error code: " <<return_code << std::endl;
      pos = ((float) position[i])*this->PosRes+this->LowerPosLim;
      return_code = CPhidgetAdvancedServo_setPosition (this->servo, i,pos); 
      if(return_code!=EPHIDGET_OK) std::cout << "Load: Error setting position[" << i <<"] Error code:" << return_code << std::endl;
      //std::cout << "Postition[" << i << "] "<< pos <<  "   Velocity[" << i << "] "<< vel <<"   Acceleration["<< i << "] "<< acc <<std::endl; 
     }
    this->phidget_access.exit();
  }catch(CException& e){
  this->phidget_access.exit();
  std::cout << e.what() << std::endl;
  throw;}
}

void CPHIDGET1061::cont_load(std::vector<float>& velocity, std::vector<float>& accel)
{

}

void CPHIDGET1061::cont_move(void)
{ 
}

void CPHIDGET1061::cont_reset(void)
{

}

void CPHIDGET1061::cont_stop(void)
{ 
  int num=this->get_num_axis();
  int i=0;
  std::vector<float> position;
  double pos=0;
  this->phidget_access.enter();
  for(i=0;i<num;i++)
  {
     if(CPhidgetAdvancedServo_getPosition (this->servo,i,&pos)!=EPHIDGET_OK) std::cout << "Stop: Error in get position" << std::endl;    
     if(CPhidgetAdvancedServo_setPosition (this->servo, i,pos)!=EPHIDGET_OK) std::cout << "Stop: Error in set position" << std::endl;    
  }
  this->phidget_access.exit();
}


CPHIDGET1061 *CPHIDGET1061::clone(CPHIDGET1061 *existing,std::string& cont_id,std::string& node_address)
{

}

CPHIDGET1061::~CPHIDGET1061()
{
if(CPhidget_close((CPhidgetHandle)this->servo)!=EPHIDGET_OK) std::cout << "Error: Close Phidget" <<std::endl;
if(CPhidget_delete((CPhidgetHandle)this->servo)!=EPHIDGET_OK) std::cout << "Error: Delete Phidget" <<std::endl;
}


