#ifndef _PHIDGET1061_H
#define _PHIDGET1061_H

#include "mutex.h"

#include "motor_control.h"
#include <stdio.h>
#include <stdlib.h>
#include "rs232.h"
#include <string>
#include "mutex.h"
#include <phidget21.h>
#define LPL -22.9921875            //Lower position limit in degrees
#define UPL 233.0                  //Upper position limit in degrees
#define POS_RESOLUTION 0.0078125   //Position resolution in degrees
#define UVL 6400.0                 //Upper velocity limit in degrees/second
#define VEL_RESOLUTION 0.390625    //Velocity resolution in degrees/second
#define UAL 320000.0               //Upper acceleration limit in degrees/second²
#define ACC_RESOLUTION 19.53125    //Acceleration resolution in degrees/second²



class CPHIDGET1061 : public CMotorControl
{
  private: 
    CPhidgetAdvancedServoHandle servo; 
    CMutex phidget_access;
    int *num_instances; 
    long int previous_position;  
    int encoder_resolution;  
    float gear_factor;  
    std::string position_event_id;
    std::string speed_event_id;
    std::string limits_event_id;
    double LowerPosLim,UpperPosLim,PosRes;
    double UpperVelLim,VelRes;
    double UpperAccLim,AccRes;
    static int AttachHandler(CPhidgetHandle SERVO, void *userptr);
    static int DetachHandler(CPhidgetHandle SERVO, void *userptr);
    static int ErrorHandler(CPhidgetHandle SERVO, void *userptr, int ErrorCode, const char *Description);
    static CPHIDGET1061 *clone(CPHIDGET1061 *existing,std::string& cont_id,std::string& node_address);
  protected:
    virtual void angles_to_controller(std::vector<float>& angles,std::vector<float>& counts); 
    virtual void speeds_to_controller(std::vector<float>& angles,std::vector<float>& counts); 
    virtual void accels_to_controller(std::vector<float>& angles,std::vector<float>& counts); 
    virtual void controller_to_angles(std::vector<float>& counts,std::vector<float>& angles) ; 
    virtual void controller_to_speeds(std::vector<float>& counts,std::vector<float>& angles) ; 
    virtual void controller_to_accels(std::vector<float>& counts,std::vector<float>& angles) ; 
    virtual void cont_set_accel_range(std::vector<float>& min, std::vector<float>& max) ; 
    virtual void cont_set_velocity_range(std::vector<float>& min, std::vector<float>& max) ; 
    virtual void cont_set_position_range(std::vector<float>& min, std::vector<float>& max) ; 
    virtual void cont_set_encoder_resolution(std::vector<int>& enc_res) ; 
    virtual void cont_set_gear_factor(std::vector<float>& gear) ; 
    virtual void cont_get_accel_range(std::vector<float>& min, std::vector<float>& max) ; 
    virtual void cont_get_velocity_range(std::vector<float>& min, std::vector<float>& max);  
    virtual void cont_get_position_range(std::vector<float>& min, std::vector<float>& max); 
    virtual void cont_get_encoder_resolution(std::vector<int>& enc_res);    
    virtual void cont_enable(std::vector<bool> &enable);
    virtual void cont_disable(std::vector<bool> &disable);
    virtual std::vector<float> cont_get_position(void);
    virtual void cont_config_position_feedback(feedback_mode mode, float rate=-1);
    virtual std::vector<float> cont_get_velocity(void);
    virtual void cont_config_velocity_feedback(feedback_mode mode, float rate=-1);
    virtual std::vector<float> cont_get_limits(void);
    virtual void cont_config_limits_feedback(feedback_mode mode, float rate=-1); 
    virtual void cont_set_gpio(int gpio_id,gpio_type type);   
    virtual void cont_write_gpio(int gpio_id,bool value);  
    virtual void cont_write_gpio(int gpio_id,float value);  
    virtual float cont_read_gpio(int gpio_id);      
    virtual void cont_home(void);  
    virtual void cont_load(std::vector<float>& position, std::vector<float>& velocity, std::vector<float>& accel);  
    virtual void cont_load(std::vector<float>& velocity, std::vector<float>& accel);  
    virtual void cont_move(void);     
    virtual void cont_reset(void);  
    virtual void cont_stop(void);  
    virtual void cont_config(void *config);
#ifdef _HAVE_XSD  
    void cont_save_config(std::string &filename);  
    void cont_load_config(std::string &filename);  
#endif
  public:
    CPHIDGET1061(std::string& cont_id);
    int serialNo, version, numMotors;
    bool attached;
    const char* DeviceType; 
    void open(void);
    int wait(int milliseconds);
    int displayProperties(void);
    void getNumMotors(int *numMotors);
    int getDeviceType(const char **deviceType);
    int getSerialNumber(int serialNo);
    int getDeviceVersion(int *version);
    std::vector<float> get_current(void);
    static int PositionChangeHandler(CPhidgetAdvancedServoHandle ADVSERVO,void *usrptr, int Index, double Value);
    static int VelocityChangeHandler(CPhidgetAdvancedServoHandle ADVSERVO,void *usrptr, int Index, double Value);
    static int CurrentChangeHandler(CPhidgetAdvancedServoHandle ADVSERVO,void *usrptr, int Index, double Value);
    void getServoType( int index, CPhidget_ServoType *servoType);
    void setServoType( int index, CPhidget_ServoType servoType);
    int setServoParameters( int index, double min_us,double max_us,double degrees,double velocity_max);
    virtual ~CPHIDGET1061(); 

};

#endif 
